@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2016 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -40,7 +40,11 @@ import time
 import os
 import sys
 import glob
-DEBUG = 0
+import logging
+
+logger = logging.getLogger(__name__)
+
+
 PLUGINS_DIR = None
 try:
     import PyMca5
@@ -67,7 +71,6 @@ except:
     PYMCA_PLUGINS_DIR = None
     pass
 
-
 class StackBase(object):
     def __init__(self):
         self._stack = DataObject.DataObject()
@@ -183,8 +186,8 @@ class StackBase(object):
                                 sys.modules[plugin].getStackPluginInstance(self)
                         self.pluginList.append(plugin)
                 except:
-                    if DEBUG:
-                        print("Problem importing module %s" % plugin)
+                    logger.debug("Problem importing module %s", plugin)
+                    if logger.getEffectiveLevel() == logging.DEBUG:
                         raise
         return len(self.pluginList)
 
@@ -256,33 +259,36 @@ class StackBase(object):
             # is not a numpy ndarray in any case
             self._tryNumpy = False
 
+        previousStackImageSize = None
+        if self._stackImageData is not None:
+            previousStackImageSize = self._stackImageData.size
+
         if self._tryNumpy and isinstance(self._stack.data, numpy.ndarray):
             self._stackImageData = numpy.sum(self._stack.data,
                                              axis=self.mcaIndex,
                                              dtype=numpy.float)
             #original ICR mca
-            if DEBUG:
-                print("(self.otherIndex, self.fileIndex) = (%d, %d)" %\
-                      (self.otherIndex, self.fileIndex))
+            logger.debug("(self.otherIndex, self.fileIndex) = (%d, %d)",
+                         self.otherIndex, self.fileIndex)
             i = max(self.otherIndex, self.fileIndex)
             j = min(self.otherIndex, self.fileIndex)
             mcaData0 = numpy.sum(numpy.sum(self._stack.data,
                                            axis=i,
                                            dtype=numpy.float), j)
         else:
-            if DEBUG:
-                t0 = time.time()
+            t0 = time.time()
             shape = self._stack.data.shape
             if self.mcaIndex in [2, -1]:
                 self._stackImageData = numpy.zeros((shape[0], shape[1]),
                                                 dtype=numpy.float)
                 mcaData0 = numpy.zeros((shape[2],), numpy.float)
                 step = 1
-                if hasattr(self._stack, "monitor"):
-                    monitor = self._stack.monitor[:]
-                    monitor.shape = shape[2]
-                else:
-                    monitor = numpy.ones((shape[2],), numpy.float)
+                # this is not the meaning of monitor
+                #if hasattr(self._stack, "monitor"):
+                #    monitor = self._stack.monitor[:]
+                #    monitor.shape = shape[2]
+                #else:
+                #    monitor = numpy.ones((shape[2],), numpy.float)
                 for i in range(shape[0]):
                     tmpData = self._stack.data[i:i+step,:,:]
                     numpy.add(self._stackImageData[i:i+step,:],
@@ -304,17 +310,26 @@ class StackBase(object):
                     mcaData0[i] = tmpData.sum()
             else:
                 raise ValueError("Unhandled case 1D index = %d" % self.mcaIndex)
-            if DEBUG:
-                print("Print dynamic loading elapsed = %f" % (time.time() - t0))
+            logger.debug("Print dynamic loading elapsed = %f", time.time() - t0)
+
+        logger.debug("__stackImageData.shape = %s", self._stackImageData.shape)
+
+        if previousStackImageSize != self._stackImageData.size:
+            self._clearPositioners()
 
-        if DEBUG:
-            print("__stackImageData.shape = ",  self._stackImageData.shape)
         calib = self._stack.info.get('McaCalib', [0.0, 1.0, 0.0])
         dataObject = DataObject.DataObject()
         dataObject.info = {"McaCalib": calib,
                            "selectiontype": "1D",
                            "SourceName": "Stack",
                            "Key": "SUM"}
+        if "McaLiveTime" in self._stack.info:
+            if hasattr(self._stack.info["McaLiveTime"], "sum"):
+                dataObject.info["McaLiveTime"] = \
+                                self._stack.info["McaLiveTime"].sum()
+            else:
+                print("Not an array. Skipping time information")
+                del dataObject.info["McaLiveTime"]
         if not hasattr(self._stack, 'x'):
             self._stack.x = None
         if self._stack.x in [None, []]:
@@ -353,6 +368,14 @@ class StackBase(object):
         for key in self.pluginInstanceDict.keys():
             self.pluginInstanceDict[key].stackUpdated()
 
+    def getStackOriginalCurve(self):
+        # TODO: Make sure copies are returned
+        x = self._mcaData0.x[0]
+        y = self._mcaData0.y[0]
+        legend = "Stack SUM"
+        info = self._mcaData0.info
+        return [x, y, legend, info]
+
     def isStackFinite(self):
         """
         Returns True if stack does not contain inf or nans
@@ -363,7 +386,7 @@ class StackBase(object):
     def handleNonFiniteData(self):
         text  = "Your data contain infinite values or nans.\n"
         text += "Pixels containing those values will be ignored."
-        print(text)
+        logger.info(text)
 
     def updateROIImages(self, ddict=None):
         if ddict is None:
@@ -388,15 +411,13 @@ class StackBase(object):
                 if len(i1):
                     i1 = min(i1)
                 else:
-                    if DEBUG:
-                        print("updateROIImages: nothing to be made")
+                    logger.debug("updateROIImages: nothing to be made")
                     return
                 i2 = numpy.nonzero(xw <= ddict['to'])[0]
                 if len(i2):
                     i2 = max(i2) + 1
                 else:
-                    if DEBUG:
-                        print("updateROIImages: nothing to be made")
+                    logger.debug("updateROIImages: nothing to be made")
                     return
                 pos = 0.5 * (ddict['from'] + ddict['to'])
                 imiddle = max(numpy.nonzero(xw <= pos)[0])
@@ -405,15 +426,13 @@ class StackBase(object):
                 if len(i2):
                     i2 = max(i2)
                 else:
-                    if DEBUG:
-                        print("updateROIImages: nothing to be made")
+                    logger.debug("updateROIImages: nothing to be made")
                     return
                 i1 = numpy.nonzero(xw <= ddict['to'])[0]
                 if len(i1):
                     i1 = min(i1) + 1
                 else:
-                    if DEBUG:
-                        print("updateROIImages: nothing to be made")
+                    logger.debug("updateROIImages: nothing to be made")
                     return
                 pos = 0.5 * (ddict['from'] + ddict['to'])
                 imiddle = min(numpy.nonzero(xw <= pos)[0])
@@ -482,12 +501,10 @@ class StackBase(object):
         self.showROIImageList(imageList, image_names=imageNames)
 
     def showOriginalImage(self):
-        if DEBUG:
-            print("showOriginalImage to be implemented")
+        logger.debug("showOriginalImage to be implemented")
 
     def showOriginalMca(self):
-        if DEBUG:
-            print("showOriginalMca to be implemented")
+        logger.debug("showOriginalMca to be implemented")
 
     def showROIImageList(self, imageList, image_names=None):
         self._ROIImageList = imageList
@@ -510,21 +527,20 @@ class StackBase(object):
             return
         mcaData = None
         goodData = numpy.isfinite(self._mcaData0.y[0].sum())
-        if DEBUG:
-            print("Stack data is not finite")
+        logger.debug("Stack data is not finite")
         if (self._selectionMask is None) and goodData:
             if normalize:
-                if DEBUG:
-                    print("Case 1")
+                logger.debug("Case 1")
                 npixels = self._stackImageData.shape[0] *\
                           self._stackImageData.shape[1] * 1.0
                 dataObject = DataObject.DataObject()
                 dataObject.info.update(self._mcaData0.info)
                 dataObject.x = [self._mcaData0.x[0]]
-                dataObject.y = [self._mcaData0.y[0] / npixels];
+                dataObject.y = [self._mcaData0.y[0] / float(npixels)]
+                if "McaLiveTime" in dataObject.info:
+                    dataObject.info["McaLiveTime"] /= float(npixels)
             else:
-                if DEBUG:
-                    print("Case 2")
+                logger.debug("Case 2")
                 dataObject = self._mcaData0
             return dataObject
 
@@ -545,16 +561,16 @@ class StackBase(object):
         npixels = actualSelectionMask.sum()
         if (npixels == 0) and goodData:
             if normalize:
-                if DEBUG:
-                    print("Case 3")
+                logger.debug("Case 3")
                 npixels = self._stackImageData.shape[0] * self._stackImageData.shape[1] * 1.0
                 dataObject = DataObject.DataObject()
                 dataObject.info.update(self._mcaData0.info)
                 dataObject.x = [self._mcaData0.x[0]]
-                dataObject.y = [self._mcaData0.y[0] / npixels]
+                dataObject.y = [self._mcaData0.y[0] / float(npixels)]
+                if "McaLiveTime" in dataObject.info:
+                    dataObject.info["McaLiveTime"] /= float(npixels)
             else:
-                if DEBUG:
-                    print("Case 4")
+                logger.debug("Case 4")
                 dataObject = self._mcaData0
             return dataObject
 
@@ -570,28 +586,22 @@ class StackBase(object):
         else:
                 arrayMask = (actualSelectionMask > 0)
 
-        if DEBUG:
-            print("Reached MCA calculation")
+        logger.debug("Reached MCA calculation")
         cleanMask = numpy.nonzero(arrayMask)
-        if DEBUG:
-            print("self.fileIndex, self.mcaIndex = %d , %d" %\
-                  (self.fileIndex, self.mcaIndex))
-        if DEBUG:
-            t0 = time.time()
+        logger.debug("self.fileIndex, self.mcaIndex = %d , %d",
+                     self.fileIndex, self.mcaIndex)
+        t0 = time.time()
         if len(cleanMask[0]) and len(cleanMask[1]):
-            if DEBUG:
-                print("USING MASK")
+            logger.debug("USING MASK")
             cleanMask = numpy.array(cleanMask).transpose()
             if self.fileIndex == 2:
                 if self.mcaIndex == 0:
                     if isinstance(self._stack.data, numpy.ndarray):
-                        if DEBUG:
-                            print("In memory case 0")
+                        logger.debug("In memory case 0")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[:, r, c]
                     else:
-                        if DEBUG:
-                            print("Dynamic loading case 0")
+                        logger.debug("Dynamic loading case 0")
                         #no other choice than to read all images
                         #for the time being, one by one
                         rMin = cleanMask[0][0]
@@ -617,13 +627,11 @@ class StackBase(object):
             elif self.fileIndex == 1:
                 if self.mcaIndex == 0:
                     if isinstance(self._stack.data, numpy.ndarray):
-                        if DEBUG:
-                            print("In memory case 2")
+                        logger.debug("In memory case 2")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[:, r, c]
                     else:
-                        if DEBUG:
-                            print("Dynamic loading case 2")
+                        logger.debug("Dynamic loading case 2")
                         #no other choice than to read all images
                         #for the time being, one by one
                         if 1:
@@ -648,13 +656,11 @@ class StackBase(object):
                                 mcaData[i] = (tmpData[0] * arrayMask).sum(dtype=numpy.float)
                 elif self.mcaIndex == 2:
                     if isinstance(self._stack.data, numpy.ndarray):
-                        if DEBUG:
-                            print("In memory case 3")
+                        logger.debug("In memory case 3")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[r, c, :]
                     else:
-                        if DEBUG:
-                            print("Dynamic loading case 3")
+                        logger.debug("Dynamic loading case 3")
                         #try to minimize access to the file
                         row_list = []
                         row_dict = {}
@@ -672,21 +678,18 @@ class StackBase(object):
             elif self.fileIndex == 0:
                 if self.mcaIndex == 1:
                     if isinstance(self._stack.data, numpy.ndarray):
-                        if DEBUG:
-                            print("In memory case 4")
+                        logger.debug("In memory case 4")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[r, :, c]
                     else:
                         raise IndexError("Dynamic loading case 4")
                 elif self.mcaIndex in [2, -1]:
                     if isinstance(self._stack.data, numpy.ndarray):
-                        if DEBUG:
-                            print("In memory case 5")
+                        logger.debug("In memory case 5")
                         for r, c in cleanMask:
                             mcaData += self._stack.data[r, c, :]
                     else:
-                        if DEBUG:
-                            print("Dynamic loading case 5")
+                        logger.debug("Dynamic loading case 5")
                         #try to minimize access to the file
                         row_list = []
                         row_dict = {}
@@ -704,17 +707,15 @@ class StackBase(object):
             else:
                 raise IndexError("File index undefined")
         else:
-            if DEBUG:
-                print("NOT USING MASK !")
+            logger.debug("NOT USING MASK !")
 
-        if DEBUG:
-            print("Mca sum elapsed = %f" % (time.time() - t0))
+        logger.debug("Mca sum elapsed = %f", time.time() - t0)
         if goodData:
             if n_nonselected < npixels:
                 mcaData = self._mcaData0.y[0] - mcaData
 
         if normalize:
-            mcaData = mcaData / npixels
+            mcaData = mcaData / float(npixels)
 
         calib = self._stack.info['McaCalib']
         dataObject = DataObject.DataObject()
@@ -722,14 +723,22 @@ class StackBase(object):
                            "selectiontype": "1D",
                            "SourceName": "Stack",
                            "Key": "Selection"}
+        if "McaLiveTime" in self._stack.info:
+            selectedPixels = actualSelectionMask > 0
+            liveTime = self._stack.info["McaLiveTime"][:]
+            liveTime.shape = actualSelectionMask.shape
+            liveTime = liveTime[selectedPixels].sum()
+            if normalize:
+                liveTime = liveTime / float(npixels)
+            dataObject.info["McaLiveTime"] = liveTime
+
         dataObject.x = [self._mcaData0.x[0]]
         dataObject.y = [mcaData]
 
         return dataObject
 
     def calculateROIImages(self, index1, index2, imiddle=None, energy=None):
-        if DEBUG:
-            print("Calculating ROI images")
+        logger.debug("Calculating ROI images")
         i1 = min(index1, index2)
         i2 = max(index1, index2)
         if imiddle is None:
@@ -761,8 +770,7 @@ class StackBase(object):
                 minImage = energy[(numpy.argmin(dataImage, axis=1) + i1)]
                 isUsingSuppliedEnergyAxis = True
             else:
-                if DEBUG:
-                    t0 = time.time()
+                t0 = time.time()
                 if self._tryNumpy and\
                    isinstance(self._stack.data, numpy.ndarray):
                     leftImage = self._stack.data[:, :, i1]
@@ -774,9 +782,8 @@ class StackBase(object):
                     maxImage = energy[numpy.argmax(dataImage, axis=2) + i1]
                     minImage = energy[numpy.argmin(dataImage, axis=2) + i1]
                     isUsingSuppliedEnergyAxis = True
-                    if DEBUG:
-                        print("Case 1 ROI image calculation elapsed = %f " %\
-                              (time.time() - t0))
+                    logger.debug("Case 1 ROI image calculation elapsed = %f ",
+                                 time.time() - t0)
                 else:
                     shape = self._stack.data.shape
                     roiImage = numpy.zeros(self._stackImageData.shape,
@@ -805,13 +812,11 @@ class StackBase(object):
                     isUsingSuppliedEnergyAxis = True
                     minImage = energy[minImage]
                     maxImage = energy[maxImage]
-                    if DEBUG:
-                        print("2 Dynamic ROI image calculation elapsed = %f " %\
-                              (time.time() - t0))
+                    logger.debug("2 Dynamic ROI image calculation elapsed = %f ",
+                                 time.time() - t0)
         elif self.fileIndex == 1:
             if self.mcaIndex == 0:
-                if DEBUG:
-                    t0 = time.time()
+                t0 = time.time()
                 if isinstance(self._stack.data, numpy.ndarray) and\
                    self._tryNumpy:
                     leftImage = self._stack.data[i1, :, :]
@@ -848,9 +853,8 @@ class StackBase(object):
                     isUsingSuppliedEnergyAxis = True
                     background = 0.5 * (i2 - i1) * (leftImage + rightImage)
                     roiImage = numpy.sum(dataImage, axis=0, dtype=numpy.float)
-                    if DEBUG:
-                        print("Case 3 ROI image calculation elapsed = %f " %\
-                              (time.time() - t0))
+                    logger.debug("Case 3 ROI image calculation elapsed = %f ",
+                                 time.time() - t0)
                 else:
                     shape = self._stack.data.shape
                     roiImage = numpy.zeros(self._stackImageData.shape,
@@ -892,12 +896,10 @@ class StackBase(object):
                     maxImage = energy[maxImage]
                     if i2 > i1:
                         background = (leftImage + rightImage) * 0.5 * (i2 - i1)
-                    if DEBUG:
-                        print("Case 4 Dynamic ROI elapsed = %f" %\
-                              (time.time() - t0))
+                    logger.debug("Case 4 Dynamic ROI elapsed = %f",
+                                 time.time() - t0)
             else:
-                if DEBUG:
-                    t0 = time.time()
+                t0 = time.time()
                 if self._tryNumpy and\
                    isinstance(self._stack.data, numpy.ndarray):
                     leftImage = self._stack.data[:, :, i1]
@@ -909,9 +911,8 @@ class StackBase(object):
                     maxImage = energy[numpy.argmax(dataImage, axis=2) + i1]
                     minImage = energy[numpy.argmin(dataImage, axis=2) + i1]
                     isUsingSuppliedEnergyAxis = True
-                    if DEBUG:
-                        print("Case 5 ROI Image elapsed = %f" %\
-                              (time.time() - t0))
+                    logger.debug("Case 5 ROI Image elapsed = %f",
+                                 time.time() - t0)
                 else:
                     shape = self._stack.data.shape
                     roiImage = numpy.zeros(self._stackImageData.shape,
@@ -938,13 +939,11 @@ class StackBase(object):
                         middleImage[i:i+step, :] += tmpData[:, :, imiddle-i1]
                         rightImage[i:i+step, :]  += tmpData[:, :,-1]
                     background = 0.5*(i2-i1)*(leftImage+rightImage)
-                    if DEBUG:
-                        print("Case 6 Dynamic ROI image calculation elapsed = %f" %\
-                                          (time.time() - t0))
+                    logger.debug("Case 6 Dynamic ROI image calculation elapsed = %f",
+                                 time.time() - t0)
         else:
             #self.fileIndex = 2
-            if DEBUG:
-                t0 = time.time()
+            t0 = time.time()
             if self.mcaIndex == 0:
                 leftImage = self._stack.data[i1]
                 middleImage = self._stack.data[imiddle]
@@ -955,9 +954,8 @@ class StackBase(object):
                 minImage = energy[numpy.argmin(dataImage, axis=0) + i1]
                 maxImage = energy[numpy.argmax(dataImage, axis=0) + i1]
                 isUsingSuppliedEnergyAxis = True
-                if DEBUG:
-                   print("Case 7 Default ROI image calculation elapsed = %f" %\
-                                          (time.time() - t0))
+                logger.debug("Case 7 Default ROI image calculation elapsed = %f",
+                             time.time() - t0)
             else:
                 leftImage = self._stack.data[:, i1, :]
                 middleImage = self._stack.data[:, imiddle, :]
@@ -968,9 +966,8 @@ class StackBase(object):
                 minImage = energy[numpy.argmin(dataImage, axis=1) + i1]
                 maxImage = energy[numpy.argmax(dataImage, axis=1) + i1]
                 isUsingSuppliedEnergyAxis = True
-                if DEBUG:
-                   print("Case 8 Default ROI image calculation elapsed = %f" %\
-                                          (time.time() - t0))
+                logger.debug("Case 8 Default ROI image calculation elapsed = %f",
+                             time.time() - t0)
 
         imageDict = {'ROI': roiImage,
                      'Maximum': maxImage,
@@ -980,13 +977,11 @@ class StackBase(object):
                      'Right': rightImage,
                      'Background': background}
         self.__ROIImageCalculationIsUsingSuppliedEnergyAxis = isUsingSuppliedEnergyAxis
-        if DEBUG:
-            print("ROI images calculated")
+        logger.debug("ROI images calculated")
         return imageDict
 
     def setSelectionMask(self, mask):
-        if DEBUG:
-            print("setSelectionMask called")
+        logger.debug("setSelectionMask called")
         goodData = numpy.isfinite(self._mcaData0.y[0].sum())
 
         if goodData:
@@ -1002,8 +997,7 @@ class StackBase(object):
             self.pluginInstanceDict[key].selectionMaskUpdated()
 
     def getSelectionMask(self):
-        if DEBUG:
-            print("getSelectionMask called")
+        logger.debug("getSelectionMask called")
         return self._selectionMask
 
     def addImage(self, image, name, info=None, replace=False, replot=True):
@@ -1056,8 +1050,7 @@ class StackBase(object):
         If just_legend is True:
             The legend of the active curve (or None) is returned.
         """
-        if DEBUG:
-            print("getActiveCurve default implementation")
+        logger.debug("getActiveCurve default implementation")
         info = {}
         info['xlabel'] = 'Channel'
         info['ylabel'] = 'Counts'
@@ -1065,13 +1058,11 @@ class StackBase(object):
         return self._mcaData0.x[0], self._mcaData0.y[0], legend, info
 
     def getGraphXLimits(self):
-        if DEBUG:
-            print("getGraphXLimits default implementation")
+        logger.debug("getGraphXLimits default implementation")
         return self._mcaData0.x[0].min(), self._mcaData0.x[0].max()
 
     def getGraphYLimits(self):
-        if DEBUG:
-            print("getGraphYLimits default implementation")
+        logger.debug("getGraphYLimits default implementation")
         return self._mcaData0.y[0].min(), self._mcaData0.y[0].max()
 
     def getStackDataObject(self):
@@ -1083,6 +1074,96 @@ class StackBase(object):
     def getStackInfo(self):
         return self._stack.info
 
+    def setPositioners(self, positioners, copy=True):
+        """Updates the "positioners" field in the stack info, after
+        checking that the provided positioners have the proper number of
+        values.
+
+        :param dict positioners: Dictionary of positioners. The keys are
+            the motor names. The values should preferably be arrays with
+            the same number of values as there are stack pixels. Scalars
+            are accepted if the positioner has a constant value.
+        :param bool copy: If *True* (default), store a copy of the data.
+            If *False*, store the original data whenever it is possible.
+            If the dictionary contains lists, they need to be converted to
+            numpy arrays and a copy is mandatory.
+        :raise: TypeError if positioners is not a dict or if any positioner
+            is not a scalar, list or numpy array.
+        :raise: RuntimeError if any positioner is a list and copy=False
+        """
+        if not isinstance(positioners, dict):
+            raise TypeError("Dictionary expected for positioners")
+
+        npixels = self.getStackOriginalImage().size
+        stackPositioners = {}
+
+        for motorName, motorValues in positioners.items():
+            if numpy.isscalar(motorValues) or \
+                    (hasattr(motorValues, "ndim") and
+                     motorValues.ndim == 0):
+                stackPositioners[motorName] = motorValues
+            elif hasattr(motorValues, "__len__") and numpy.isscalar(motorValues[0]):
+                # list: convert to numpy array before storing in info
+                numMotorValues = len(motorValues)
+                if numMotorValues == npixels:
+                    stackPositioners[motorName] = numpy.array(motorValues)
+            elif hasattr(motorValues, "size"):
+                # numpy array
+                numMotorValues = motorValues.size
+                if numMotorValues == npixels:
+                    stackPositioners[motorName] = numpy.array(motorValues,
+                                                              copy=copy)
+            else:
+                raise TypeError(
+                        "Wrong type for positioner %s. " % motorName +
+                        "Valid types are numpy arrays, scalars or list of scalars.")
+
+        if len(stackPositioners) != len(positioners):
+            ignored_motors = list(set(positioners.keys()) -
+                                  set(stackPositioners.keys()))
+            logger.debug("Ignored motors due to mismatch in number of values: %s",
+                          ', '.join(ignored_motors))
+
+        self._stack.info["positioners"] = stackPositioners
+
+    def _clearPositioners(self):
+        """Removes the "positioners" field in the stack info"""
+        if "positioners" in self._stack.info:
+            self._stack.info["positioners"] = {}
+
+    def getPositionersFromIndex(self, index):
+        """Return the value of all positioners for the spectrum identified by
+        its 1D index.
+
+        If positioners are stored as 1D arrays ``a``, the value returned
+        is simply ``a[index]``.
+
+        If positioners are stored as 2D arrays, the index is applied to the
+        flattened array.
+
+        :param int index: Index of spectrum for which the the positioners
+            value is to be returned.
+        :return: dictionary of positioners values whose keys are the motor
+            name
+        :rtype: dict
+        """
+        positioners = self._stack.info.get("positioners", {})
+        positionersAtIdx = {}
+
+        if index >= self.getStackOriginalImage().size or index < 0:
+            raise IndexError("index out of bounds")
+
+        for motorName, motorValues in positioners.items():
+            # assert numpy.isscalar(motorValues) or hasattr(motorValues, "shape")
+            if numpy.isscalar(motorValues):
+                positionersAtIdx[motorName] = motorValues
+            elif len(motorValues.shape) == 1:
+                positionersAtIdx[motorName] = motorValues[index]
+            else:
+                positionersAtIdx[motorName] = motorValues.reshape((-1,))[index]
+
+        return positionersAtIdx
+
 
 def test():
     #create a dummy stack
