@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2015 V.A. Sole, ESRF - D. Dale CHESS
+# Copyright (C) 2004-2019 V.A. Sole, ESRF - D. Dale CHESS
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -27,12 +27,29 @@ __author__ = "Darren Dale (CHESS) & V.A. Sole (ESRF)"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
+
 import sys
 import os
 import posixpath
 import gc
 import re
 from operator import itemgetter
+import logging
+
+import h5py
+import weakref
+
+try:
+    from silx.io import is_group
+    from silx.io import open as h5open
+    import logging
+    logging.getLogger("silx.io.fabioh5").setLevel(logging.CRITICAL)
+except ImportError:
+    def is_group(node):
+        return isinstance(node, h5py.Group)
+
+    def h5open(filename):
+        return h5py.File(filename, "r")
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
 safe_str = qt.safe_str
@@ -43,13 +60,12 @@ else:
     def MyQVariant(x=None):
         return x
 
-import h5py
-phynx = h5py
-import weakref
 
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 QVERSION = qt.qVersion()
 
+
 #sorting method
 def h5py_sorting(object_list):
     sorting_list = ['start_time', 'end_time', 'name']
@@ -67,6 +83,7 @@ def h5py_sorting(object_list):
         posixNames = [item[1].name for item in object_list]
     except AttributeError:
         # Typical of broken external links
+        _logger.debug("HDF5Widget: Cannot get posixNames")
         return object_list
 
     # This implementation only sorts entries
@@ -74,10 +91,12 @@ def h5py_sorting(object_list):
         return object_list
 
     sorting_key = None
-    for key in sorting_list:
-        if key in object_list[0][1]:
-            sorting_key = key
-            break
+    if hasattr(object_list[0][1], "items"):
+        for key in sorting_list:
+            if key in [x[0] for x in object_list[0][1].items()]:
+                sorting_key = key
+                break
+
     if sorting_key is None:
         if 'name' in sorting_list:
             sorting_key = 'name'
@@ -86,7 +105,7 @@ def h5py_sorting(object_list):
 
     try:
         if sorting_key != 'name':
-            sorting_list = [(o[1][sorting_key].value, o)
+            sorting_list = [(o[1][sorting_key][()], o)
                            for o in object_list]
             sorted_list = sorted(sorting_list, key=itemgetter(0))
             return [x[1] for x in sorted_list]
@@ -105,7 +124,7 @@ def h5py_sorting(object_list):
         return object_list
 
 def _get_number_list(txt):
-    rexpr = '[/a-zA-Z:-]'
+    rexpr = '[/a-zA-Z:_-]'
     nbs= [float(w) for w in re.split(rexpr, txt) if w not in ['',' ']]
     return nbs
 
@@ -182,11 +201,7 @@ class H5NodeProxy(object):
             # obtaining the lock here is necessary, otherwise application can
             # freeze if navigating tree while data is processing
             if 1: #with self.file.plock:
-                items = self.getNode(self.name).items()
-                # previous line seems to retrieve an iterator under Python 3.3
-                if (sys.version > '3.2') and\
-                   (h5py.version.version > '2.1.1'):
-                    items = list(items)
+                items = list(self.getNode(self.name).items())
                 if posixpath.dirname(self.name) == "/":
                     # top level item
                     doit = True
@@ -209,10 +224,13 @@ class H5NodeProxy(object):
                 except:
                     #one cannot afford any error, so I revert to the old
                     # method where values where used instead of items
-                    if DEBUG:
+                    if _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
                     else:
-                        tmpList = list(self.getNode(self.name).values())
+                        # tmpList = list(self.getNode(self.name).values())
+                        # spech5 does not implement values() prior to silx 0.6.0
+                        tmpList = list(map(self.getNode(self.name).__getitem__,
+                                           self.getNode(self.name).keys()))
                         finalList = tmpList
                     for i in range(len(finalList)):
                         finalList[i]._posixPath = posixpath.join(self.name,
@@ -266,6 +284,13 @@ class H5NodeProxy(object):
     def parent(self):
         return self._parent
 
+    #@property
+    #def attrs(self):
+    #    return self._attrs
+
+    @property
+    def color(self):
+        return self._color
 
     def __init__(self, ffile, node, parent=None, path=None):
         if 1:#with ffile.plock:
@@ -282,16 +307,27 @@ class H5NodeProxy(object):
                 self._name = posixpath.basename(node.name)
             """
             self._type = type(node).__name__
-            self._hasChildren = isinstance(node, h5py.Group)
+
+            self._hasChildren = is_group(node)
+            #self._attrs = []
+            self._color = qt.QColor(qt.Qt.black)
             if hasattr(node, 'attrs'):
                 attrs = list(node.attrs)
                 for cname in ['class', 'NX_class']:
                     if cname in attrs:
+                        nodeattr = node.attrs[cname]
                         if sys.version <'3.0':
-                            _type = "%s" % node.attrs[cname]
+                            _type = "%s" % nodeattr
+                        elif hasattr(nodeattr, "decode"):
+                            _type = nodeattr.decode('utf=8')
                         else:
-                            _type = node.attrs[cname].decode('utf=8')
+                            _type = "%s" % nodeattr
                         self._type = _type
+                        if _type in ["NXdata"]:
+                            self._color = qt.QColor(qt.Qt.blue)
+                        elif ("default" in attrs):
+                            self._color = qt.QColor(qt.Qt.blue)                        
+                        #self._attrs = attrs
                         break
                         #self._type = _type[2].upper() + _type[3:]
             self._children = []
@@ -369,36 +405,50 @@ class FileModel(qt.QAbstractItemModel):
         return 4
 
     def data(self, index, role):
-        if role != qt.Qt.DisplayRole:
-            return MyQVariant()
-        item = self.getProxyFromIndex(index)
-        column = index.column()
-        if column == 0:
-            if isinstance(item, H5FileProxy):
-                return MyQVariant(os.path.basename(item.file.filename))
-            else:
-                return MyQVariant(posixpath.basename(item.name))
-        if column == 1:
-            showtitle = True
-            if showtitle:
-                if hasattr(item, 'type'):
-                    if item.type in ["Entry", "NXentry"]:
-                        children = item.children
-                        names = [posixpath.basename(o.name) for o in children]
-                        if "title" in names:
-                            idx = names.index("title")
-                            if len(children[idx].getNode().shape):
-                                #stored as an array of strings!!!
-                                #return just the first item
-                                return MyQVariant("%s" % children[idx].getNode().value[0])
-                            else:
-                                #stored as a string
-                                return MyQVariant("%s" % children[idx].getNode().value)
-            return MyQVariant(item.type)
-        if column == 2:
-            return MyQVariant(item.shape)
-        if column == 3:
-            return MyQVariant(item.dtype)
+        if role == qt.Qt.DisplayRole:
+            item = self.getProxyFromIndex(index)
+            column = index.column()
+            if column == 0:
+                if isinstance(item, H5FileProxy):
+                    return MyQVariant(os.path.basename(item.file.filename))
+                else:
+                    if hasattr(item, "name"):
+                        return MyQVariant(posixpath.basename(item.name))
+                    else:
+                        # this can only happen with the root
+                        return MyQVariant("/")
+            if column == 1:
+                showtitle = True
+                if showtitle:
+                    if hasattr(item, 'type'):
+                        if item.type in ["Entry", "NXentry"]:
+                            children = item.children
+                            names = [posixpath.basename(o.name) for o in children]
+                            if "title" in names:
+                                idx = names.index("title")
+                                if len(children[idx].getNode().shape):
+                                    #stored as an array of strings!!!
+                                    #return just the first item
+                                    return MyQVariant("%s" % children[idx].getNode()[()][0])
+                                else:
+                                    #stored as a string
+                                    return MyQVariant("%s" % children[idx].getNode()[()])
+                return MyQVariant(item.type)
+            if column == 2:
+                return MyQVariant(item.shape)
+            if column == 3:
+                return MyQVariant(item.dtype)
+        elif role == qt.Qt.ForegroundRole:
+            item = self.getProxyFromIndex(index)
+            column = index.column()
+            if column == 0:
+                if hasattr(item, "color"):
+                    return MyQVariant(qt.QColor(item.color))
+        elif role == qt.Qt.ToolTipRole:
+            item = self.getProxyFromIndex(index)
+            if hasattr(item, "color"):
+                if item.color == qt.Qt.blue:
+                    return MyQVariant("Item has a double click NXdata associated action")
         return MyQVariant()
 
     def getNodeFromIndex(self, index):
@@ -467,7 +517,7 @@ class FileModel(qt.QAbstractItemModel):
                 ddict['filename'] = filename
                 self.sigFileUpdated.emit(ddict)
                 return item.file
-        phynxFile = phynx.File(filename, 'r')
+        phynxFile = h5open(filename)
         if weakreference:
             def phynxFileInstanceDistroyed(weakrefObject):
                 idx = self.rootItem._identifiers.index(id(weakrefObject))
@@ -532,8 +582,20 @@ class FileModel(qt.QAbstractItemModel):
         self.sigFileAppended.emit(ddict)
 
     def clear(self):
-        self.reset()
-
+        _logger.debug("Clear called")
+        # reset is considered obsolete under Qt 5.
+        if hasattr(self, "reset"):
+            self.reset()
+        else:
+            rootItem = self.rootItem
+            self.beginResetModel()
+            #for idx in range(len(rootItem._children)):
+            #    child = rootItem._children[idx]
+            #    child.clearChildren()
+            #    del self._idMap[id(child)]
+            #    rootItem.deleteChild(child)
+            rootItem.children.clear()
+            self.endResetModel()
 
 class FileView(qt.QTreeView):
 
@@ -572,6 +634,7 @@ class FileView(qt.QTreeView):
                         break
         self.doItemsLayout()
 
+
 class HDF5Widget(FileView):
     def __init__(self, model, parent=None):
         FileView.__init__(self, model, parent)
@@ -611,7 +674,7 @@ class HDF5Widget(FileView):
                 self.emitSignal(event, modelIndex)
 
     def itemActivated(self, modelIndex):
-        event ="itemActivated"
+        event = "itemActivated"
         self.emitSignal(event, modelIndex)
 
     def itemClicked(self, modelIndex):
@@ -622,10 +685,16 @@ class HDF5Widget(FileView):
         event ="itemDoubleClicked"
         self.emitSignal(event, modelIndex)
 
+    def selectionChanged(self, selected, deselected):
+        super(HDF5Widget, self).selectionChanged(selected, deselected)
+        event = "itemSelectionChanged"
+        modelIndex = self.currentIndex()
+        self.emitSignal(event, modelIndex)
+
     def emitSignal(self, event, modelIndex):
         if self.model() is None:
             return
-        item  = self.model().getProxyFromIndex(modelIndex)
+        item = self.model().getProxyFromIndex(modelIndex)
         if QVERSION > "5":
             # prevent crash clicking on empty space
             if not hasattr(item, "file"):
@@ -638,6 +707,7 @@ class HDF5Widget(FileView):
         ddict['type']  = item.type
         ddict['dtype'] = item.dtype
         ddict['shape'] = item.shape
+        ddict['color'] = item.color
         ddict['mouse'] = self._lastMouse * 1
         self.sigHDF5WidgetSignal.emit(ddict)
 
@@ -659,58 +729,192 @@ class HDF5Widget(FileView):
                 entryList.append((entry, item.file.filename))
         return entryList
 
-def getDatasetDialog(filename=None, value=False, message=None):
-    """
-    Simple dialog to select a dataset via a double click on the tree
 
-    :param filename: Name of the HDF5 file
-    :param value: If True returns dataset value instead of just the dataset
-    """
-    if filename is None:
-        from PyMca5.PyMca import PyMcaFileDialogs
-        fileTypeList = ['HDF5 Files (*.h5 *.nxs *.hdf)',
-                        'HDF5 Files (*)']
-        message = "Open HDF5 file"
-        filenamelist, ffilter = PyMcaFileDialogs.getFileList(parent=None,
-                                    filetypelist=fileTypeList,
-                                    message=message,
-                                    getfilter=True,
-                                    single=True,
-                                    currentfilter=None)
-        if len(filenamelist) < 1:
-            return None
-        filename = filenamelist[0]
-    if message is None:
-        message = 'Select your item by a double click'
-    hdf5Dialog = qt.QDialog()
-    hdf5Dialog.setWindowTitle(message)
-    hdf5Dialog.mainLayout = qt.QVBoxLayout(hdf5Dialog)
-    hdf5Dialog.mainLayout.setContentsMargins(0, 0, 0, 0)
-    hdf5Dialog.mainLayout.setSpacing(0)
-    fileModel = FileModel()
-    fileView = HDF5Widget(fileModel)
-    hdf5File = fileModel.openFile(filename, "r")
-    def _hdf5WidgetSlot(ddict):
+class Hdf5SelectionDialog(qt.QDialog):
+    """Dialog widget to select a HDF5 item in a file.
+
+    It is composed of a :class:`HDF5Widget` tree view,
+    and two buttons Ok and Cancel.
+
+    When the dialog's execution is ended with a click on the OK button,
+    or with a double-click on an item of the proper type, the URI of
+    the selected item will be available in attribute :attr:`selectedItemUri`.
+
+    If the user clicked cancel or closed the dialog
+    without selecting an item, :attr:`selectedItemUri` will be None."""
+    datasetTypes = ['dataset',
+                    'spech5dataset', 'spech5linktodataset',  # spech5
+                    'framedata', 'rawheaderdata']            # fabioh5
+
+    def __init__(self, parent=None,
+                 filename=None, message=None, itemtype="any"):
+        """
+
+        :param filename: Name of the HDF5 file
+        :param value: If True returns dataset value instead of just the dataset.
+            This must be False if itemtype is not "dataset".
+        :param str itemtype: "dataset" or "group" or "any" (default)
+        """
+        message = message if message is not None else 'Select your item'
+        self.itemtype = itemtype if itemtype is not None else "any"
+
+        if self.itemtype not in ["any", "dataset", "group"]:
+            raise AttributeError(
+                    "Invalid itemtype %s, should be 'group', 'dataset' or 'any'" % itemtype)
+
+        if filename is None:
+            filename = _getFilenameDialog()
+        if filename is None:
+            raise IOError("No filename specified")
+
+        qt.QDialog.__init__(self, parent)
+
+        self.setWindowTitle(message)
+        mainLayout = qt.QVBoxLayout(self)
+        mainLayout.setContentsMargins(0, 0, 0, 0)
+        mainLayout.setSpacing(0)
+        self.fileModel = FileModel()
+        self.fileView = HDF5Widget(self.fileModel)
+        self.filename = filename
+
+        self.fileView.sigHDF5WidgetSignal.connect(self._hdf5WidgetSlot)
+
+        mainLayout.addWidget(self.fileView)
+
+        buttonContainer = qt.QWidget(self)
+        buttonContainerLayout = qt.QHBoxLayout(buttonContainer)
+        mainLayout.setContentsMargins(0, 0, 0, 0)
+        mainLayout.setSpacing(0)
+
+        self.okb = qt.QPushButton("OK", buttonContainer)
+        cancelb = qt.QPushButton("Cancel", buttonContainer)
+
+        self.okb.clicked.connect(self.onOk)
+        self.okb.setEnabled(False)    # requires item to be clicked or activated
+        cancelb.clicked.connect(self.reject)
+
+        buttonContainerLayout.addWidget(self.okb)
+        buttonContainerLayout.addWidget(cancelb)
+
+        mainLayout.addWidget(buttonContainer)
+
+        self.resize(400, 200)
+        self.selectedItemUri = None
+        """URI of selected HDF5 item, with format 'filename::item_name'
+        """
+
+        self._lastEvent = None
+        """Dictionary with info about latest event"""
+
+    def _hdf5WidgetSlot(self, ddict):
+        self._lastEvent = ddict
+        eventType = ddict['type'].lower()
+
+        isExpectedType = self.itemtype.lower() == "any" or \
+                (eventType in self.datasetTypes and self.itemtype == "dataset") or \
+                (eventType not in self.datasetTypes and self.itemtype == "group")
+
+        if isExpectedType:
+            self.okb.setEnabled(True)
+        else:
+            self.okb.setEnabled(False)
+
         if ddict['event'] == "itemDoubleClicked":
-            if ddict['type'].lower() in ['dataset']:
-                hdf5Dialog._hdf5Datatset = ddict['name']
-                hdf5Dialog.accept()    
-    fileView.sigHDF5WidgetSignal.connect(_hdf5WidgetSlot)
-    hdf5Dialog.mainLayout.addWidget(fileView)
-    hdf5Dialog.resize(400, 200)
+            if isExpectedType:
+                self.selectedItemUri = ddict['file'] + "::" + ddict['name']
+                self.accept()
+
+    def onOk(self):
+        self.selectedItemUri = self._lastEvent['file'] + "::" + self._lastEvent['name']
+        self.accept()
+
+    def exec_(self):
+        with h5open(self.filename) as hdf5File:
+            self.fileModel.appendPhynxFile(hdf5File, weakreference=True)
+            ret = qt.QDialog.exec_(self)
+        return ret
+
+
+def _getFilenameDialog():
+    """Open a dialog to select a file in a filesystem tree view.
+    Return the selected filename."""
+    from PyMca5.PyMcaGui.io import PyMcaFileDialogs
+    fileTypeList = ['HDF5 Files (*.h5 *.nxs *.hdf)',
+                    'HDF5 Files (*)']
+    message = "Open HDF5 file"
+    filenamelist, ffilter = PyMcaFileDialogs.getFileList(parent=None,
+                                filetypelist=fileTypeList,
+                                message=message,
+                                getfilter=True,
+                                single=True,
+                                currentfilter=None)
+    if len(filenamelist) < 1:
+        return None
+    return filenamelist[0]
+
+
+def getDatasetValueDialog(filename=None, message=None):
+    """Open a dialog to select a dataset in a HDF5 file.
+    Return the value of the dataset.
+
+    If the dataset selection was cancelled, None is returned.
+
+    :param str filename: HDF5 file path. If None, a file dialog
+        is used to select the file.
+    :param str message: Message used as window title for dialog
+    :return: HDF5 dataset as numpy array, or None
+    """
+    hdf5Dialog = Hdf5SelectionDialog(None, filename, message,
+                                     "dataset")
     ret = hdf5Dialog.exec_()
     if not ret:
         return None
-    dataset = hdf5Dialog._hdf5Datatset
-    hdf5Dialog = None
-    if value:
-        data = hdf5File[dataset].value
-        # is it dangerous to close the file?
-        hdf5File.close()
-    else:
-        data = hdf5File[dataset]
+
+    selectedHdf5Uri = hdf5Dialog.selectedItemUri
+
+    with h5open(filename) as hdf5File:
+        hdf5Item = hdf5File[selectedHdf5Uri.split("::")[-1]]
+        data = hdf5Item.value
+
     return data
 
+
+def getDatasetDialog(filename=None, value=False, message=None):
+    # function kept for backward compatibility, in case someone
+    # uses it with value=False outside PyMca5
+    if value:
+        return getDatasetValueDialog(filename, message)
+
+    hdf5Dialog = Hdf5SelectionDialog(None, filename, message,
+                                     "dataset")
+    ret = hdf5Dialog.exec_()
+    if not ret:
+        return None
+    selectedHdf5Uri = hdf5Dialog.selectedItemUri
+    hdf5File = h5open(filename)
+    return hdf5File[selectedHdf5Uri.split("::")[-1]]
+
+
+def getGroupNameDialog(filename=None, message=None):
+    """Open a dialog to select a group in a HDF5 file.
+    Return the name of the group.
+
+    :param str filename: HDF5 file path. If None, a file dialog
+        is used to select the file.
+    :param str message: Message used as window title for dialog
+    :return: HDF5 group name
+    """
+    hdf5Dialog = Hdf5SelectionDialog(None, filename, message,
+                                     "group")
+    ret = hdf5Dialog.exec_()
+    if not ret:
+        return None
+
+    selectedHdf5Uri = hdf5Dialog.selectedItemUri
+
+    return selectedHdf5Uri.split("::")[-1]
+
+
 if __name__ == "__main__":
     if len(sys.argv) < 2:
         print("Usage:")
@@ -722,7 +926,7 @@ if __name__ == "__main__":
     phynxFile = fileModel.openFile(sys.argv[1])
     def mySlot(ddict):
         print(ddict)
-        if ddict['type'].lower() in ['dataset']:
+        if ddict['type'].lower() in Hdf5SelectionDialog.datasetTypes:
             print(phynxFile[ddict['name']].dtype, phynxFile[ddict['name']].shape)
     fileView.sigHDF5WidgetSignal.connect(mySlot)
     fileView.show()
