@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2016 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -80,7 +80,8 @@ class PlotWindow(PlotWidget.PlotWidget):
         self.setWindowType(None)      # None, "SCAN", "MCA"
         self._initIcons()
         self._buildToolBar(kw)
-        self.setIconSize(qt.QSize(16, 16))
+        if "silx" not in sys.modules:
+            self.setIconSize(qt.QSize(16, 16))
         self._toggleCounter = 0
         self._keepDataAspectRatioFlag = False
         self.gridLevel = 0
@@ -156,7 +157,7 @@ class PlotWindow(PlotWidget.PlotWidget):
             self._yPos.setFixedWidth(self._yPos.fontMetrics().width('##############'))
             self.graphBottomLayout.addWidget(self._yPos)
             self.graphBottomLayout.addWidget(qt.HorizontalSpacer(self.graphBottom))
-        widget.layout().addWidget(self.graphBottom)
+        widget.layout().addWidget(self.graphBottom, 0)
 
     def setPrintMenu(self, menu):
         self._printMenu = menu
@@ -1268,8 +1269,8 @@ class PlotWindow(PlotWidget.PlotWidget):
                 idx = nonzero((fromData <= xproc) &\
                                    (xproc <= toData))[0]
                 if len(idx):
-                    xw = x[idx]
-                    yw = y[idx]
+                    xw = xproc[idx]
+                    yw = yproc[idx]
                     rawCounts = yw.sum(dtype=numpy.float)
                     deltaX = xw[-1] - xw[0]
                     deltaY = yw[-1] - yw[0]
@@ -1473,7 +1474,10 @@ class PlotWindow(PlotWidget.PlotWidget):
             os.remove(filename)
         if filterused[0].upper() == "WIDGET":
             fformat = filename[-3:].upper()
-            pixmap = qt.QPixmap.grabWidget(self)
+            if hasattr(qt.QPixmap,"grabWidget"):
+                pixmap = qt.QPixmap.grabWidget(self)
+            else:
+                pixmap = self.grab()
             if not pixmap.save(filename, fformat):
                 msg = qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Critical)
@@ -1623,12 +1627,12 @@ class PlotWindow(PlotWidget.PlotWidget):
         for idx in range(0, length, 16):
             if idx+15 < length:
                 for i in range(0, 16):
-                    tmpstr += "%.7g " % data[idx+i]
+                    tmpstr += "%.8g " % data[idx+i]
                 if idx+16 != length:
                     tmpstr += "\\"
             else:
                 for i in range(idx, length):
-                    tmpstr += "%.7g " % data[i]
+                    tmpstr += "%.8g " % data[i]
             tmpstr += "\n"
         return tmpstr
 
