@@ -28,7 +28,8 @@ __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
-import os
+import logging
+import traceback
 from PyMca5.PyMcaGui import PyMcaQt as qt
 DEBUG = 0
 __revision__="$Revision: 1.7 $"
@@ -39,6 +40,14 @@ __revision__="$Revision: 1.7 $"
 
 QTVERSION = qt.qVersion()
 
+_logger = logging.getLogger(__name__)
+_logger.warning("%s is deprecated, you are advised to use "
+                "silx.gui.widgets.PrintPreview instead",
+                __name__)
+
+for line in traceback.format_stack(limit=3):
+    _logger.warning(line.rstrip())
+
 
 ################################################################################
 ##################             PyMcaPrintPreview               ###################
@@ -129,7 +138,7 @@ class PyMcaPrintPreview(qt.QDialog):
 
     def _buildToolbar(self):
         # --- command buttons
-        buttonSize = 65
+        # buttonSize = 65
         toolBar = qt.QWidget(self)
         # a layout for the toolbar
         toolsLayout = qt.QHBoxLayout(toolBar)
@@ -137,27 +146,26 @@ class PyMcaPrintPreview(qt.QDialog):
         toolsLayout.setSpacing(0)
 
         # Margin
-        """
-        marginLabel = qt.QLabel("Margins:", toolBar)
-        if QTVERSION < '4.0.0':
-            self.marginSpin = qt.QSpinBox(0, 50, 10, toolBar)
-        else:
-            self.marginSpin = qt.QSpinBox(toolBar)
-            self.marginSpin.setRange(0, 50)
-            self.marginSpin.setSingleStep(10)
-        self.marginSpin.valueChanged[int].connect( \
-                 self.__marginChanged)
-        """
+        # marginLabel = qt.QLabel("Margins:", toolBar)
+        # if QTVERSION < '4.0.0':
+        #     self.marginSpin = qt.QSpinBox(0, 50, 10, toolBar)
+        # else:
+        #     self.marginSpin = qt.QSpinBox(toolBar)
+        #     self.marginSpin.setRange(0, 50)
+        #     self.marginSpin.setSingleStep(10)
+        # self.marginSpin.valueChanged[int].connect( \
+        #          self.__marginChanged)
+
         # Scale / Zoom
-        scaleLabel = qt.QLabel("Zoom:", toolBar)
-        scaleCombo = qt.QComboBox(toolBar)
-        self.scaleValues = [20, 40, 60, 80, 100, 150, 200]
-
-        for scale in self.scaleValues:
-            scaleCombo.addItem("%3d %%"%scale)
-
-        self.scaleCombo = scaleCombo
-        self.scaleCombo.activated[int].connect(self.__scaleChanged)
+        # scaleLabel = qt.QLabel("Zoom:", toolBar)
+        # scaleCombo = qt.QComboBox(toolBar)
+        # self.scaleValues = [20, 40, 60, 80, 100, 150, 200]
+        #
+        # for scale in self.scaleValues:
+        #     scaleCombo.addItem("%3d %%"%scale)
+        #
+        # self.scaleCombo = scaleCombo
+        # self.scaleCombo.activated[int].connect(self.__scaleChanged)
 
         hideBut   = qt.QPushButton("Hide", toolBar)
         #hideBut.setFixedWidth(buttonSize-10)
@@ -642,6 +650,7 @@ class GraphicsResizeRectItem(qt.QGraphicsRectItem):
         self.__point0 = self.pos()
         parent = self.parentItem()
         scene  = self.scene()
+        scene.clearSelection()
         rect = parent.rect()
         self._x = rect.x()
         self._y = rect.y()
