@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2016 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -32,6 +32,7 @@ import sys
 import os
 import time
 import subprocess
+import logging
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
 
@@ -47,6 +48,7 @@ except ImportError:
 from PyMca5.PyMcaIO import ConfigDict
 from PyMca5.PyMcaPhysics.xrf import McaAdvancedFitBatch
 from PyMca5.PyMcaGui.physics.xrf import QtMcaAdvancedFitReport
+from PyMca5.PyMcaGui.io import PyMcaFileDialogs
 from PyMca5.PyMcaCore import EdfFileLayer
 from PyMca5.PyMcaCore import SpecFileLayer
 from PyMca5.PyMcaGui import IconDict
@@ -57,7 +59,8 @@ from PyMca5.PyMcaCore import PyMcaDirs
 from PyMca5.PyMcaCore import PyMcaBatchBuildOutput
 
 ROIWIDTH = 100.
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 class McaBatchGUI(qt.QWidget):
     def __init__(self,parent=None,name="PyMca batch fitting",fl=None,
@@ -73,15 +76,17 @@ class McaBatchGUI(qt.QWidget):
         self._processList = []
         self._selection = None
         self.__build(actions)
-        if filelist is None: filelist = []
-        self.inputDir   = None
+        if filelist is None:
+            filelist = []
+        self.inputDir = None
+        self.inputFilter = None
         self.outputDir  = None
         if outputdir is not None:
             if os.path.exists(outputdir):
                 self.outputDir = outputdir
             else:
                 qt.QMessageBox.information(self, "INFO",
-                    "Directory %s does not exists\nUsing %s"% (outputdir, self.outputDir))
+                    "Directory %s does not exist\nUsing %s"% (outputdir, self.outputDir))
 
         self.configFile = None
         self.setFileList(filelist)
@@ -192,8 +197,7 @@ class McaBatchGUI(qt.QWidget):
         if QTVERSION < '4.0.0':
             palette.setDisabled(palette.active())
         else:
-            if DEBUG:
-                print("palette set disabled")
+            _logger.debug("palette set disabled")
         self.__imgBox.setChecked(True)
         self.__imgBox.setEnabled(False)
         vbox2.l.addWidget(self.__imgBox)
@@ -460,7 +464,8 @@ class McaBatchGUI(qt.QWidget):
                         dialog.mainLayout.setContentsMargins(0, 0, 0, 0)
                         dialog.mainLayout.setSpacing(0)
                         datasource = NexusDataSource.NexusDataSource(filelist[0])
-                        nexusWidget = QNexusWidget.QNexusWidget(dialog)
+                        nexusWidget = QNexusWidget.QNexusWidget(dialog,
+                                                                buttons=True)
                         nexusWidget.buttons.hide()
                         nexusWidget.setDataSource(datasource)
                         button = qt.QPushButton(dialog)
@@ -498,7 +503,7 @@ class McaBatchGUI(qt.QWidget):
                         #self._hdf5Widget.selectionWidgetsDict['m'].hide()
                         if self._hdf5Widget.isHidden():
                             self._hdf5Widget.show()
-                elif filelist[0][-3:].lower() in ['.h5', 'nxs', 'hdf']:
+                elif filelist[0][-3:].lower() in ['.h5', 'nxs', 'hdf', 'hdf5']:
                             text = "Warning, this looks as an HDF5 file "
                             text += "but you do not have HDF5 support."
                             self.showMessage(text)
@@ -549,7 +554,7 @@ class McaBatchGUI(qt.QWidget):
         for ffile in filelist:
             if not os.path.exists(ffile):
                 qt.QMessageBox.critical(self, "ERROR",
-                                    'File %s\ndoes not exists' % ffile)
+                                    'File %s\ndoes not exist' % ffile)
                 if QTVERSION < '4.0.0':
                     self.raiseW()
                 else:
@@ -565,7 +570,7 @@ class McaBatchGUI(qt.QWidget):
         for configfile in configfileList:
             if not os.path.exists(configfile):
                 qt.QMessageBox.critical(self,
-                             "ERROR",'File %s\ndoes not exists' % configfile)
+                             "ERROR",'File %s\ndoes not exist' % configfile)
                 if QTVERSION < '4.0.0':
                     self.raiseW()
                 else:
@@ -628,49 +633,24 @@ class McaBatchGUI(qt.QWidget):
         if not os.path.exists(self.inputDir):
             self.inputDir =  os.getcwd()
         wdir = self.inputDir
-        filedialog = qt.QFileDialog(self)
-        filedialog.setWindowTitle("Open a set of files")
-        filedialog.setDirectory(wdir)
-        filedialog.setModal(1)
-        filedialog.setFileMode(filedialog.ExistingFiles)
-
+        wfilter = self.inputFilter
         filetypes  = "McaFiles (*.mca)\nEdfFiles (*.edf)\n"
         if HDF5SUPPORT:
-            filetypes += "HDF5 (*.nxs *.h5 *.hdf)\n"
+            filetypes += "HDF5 (*.nxs *.h5 *.hdf *.hdf5)\n"
         filetypes += "SpecFiles (*.spec)\nSpecFiles (*.dat)\nAll files (*)"
-        #if (QTVERSION < '4.3.0') and sys.platform == "win32":
-        if False and PyMcaDirs.nativeFileDialogs:
-            #windows file dialogs have difficulties when dealing with
-            #thousands of files, I do not know if other platforms too
-            try:
-                # API 1
-                filelist = qt.QFileDialog.getOpenFileNames(self,
-                            "Open a set of files",
-                            wdir,
-                            filetypes,
-                            None)    #This should be the last file filter used
-            except:
-                # API 2
-                filelist, andFilter = \
-                            qt.QFileDialog.getOpenFileNamesAndFilter(self,
-                            "Open a set of files",
-                            wdir,
-                            filetypes,
-                            None)    #This should be the last file filter used
-        else:
-            if hasattr(filedialog, "setFilters"):
-                filedialog.setFilters(filetypes.split("\n"))
-            else:
-                filedialog.setNameFilters(filetypes.split("\n"))
-            ret = filedialog.exec_()
-            if  ret == qt.QDialog.Accepted:
-                filelist=filedialog.selectedFiles()
-            else:
-                self.raise_()
-                return
+        filetypelist = filetypes.split("\n")
+        filelist, filefilter = PyMcaFileDialogs.getFileList(self,
+                                                filetypelist=filetypelist,
+                                                message="Open a set of files",
+                                                currentdir=wdir,
+                                                mode="OPEN",
+                                                getfilter=True,
+                                                single=False,
+                                                currentfilter=wfilter)
+
         if len(filelist):
-            filelist = [qt.safe_str(x) for x in filelist]
             self.setFileList(filelist)
+            self.inputFilter = filefilter
         self.raise_()
 
     def browseConfig(self):
@@ -678,56 +658,20 @@ class McaBatchGUI(qt.QWidget):
         if not os.path.exists(self.inputDir):
             self.inputDir =  os.getcwd()
         wdir = self.inputDir
-        if QTVERSION < '4.0.0':
-            filename = qt.QFileDialog(self,"Open a new fit config file",1)
-            filename.setMode(filename.ExistingFiles)
-            filename.setDir(wdir)
-        else:
-            filename = qt.QFileDialog(self)
-            filename.setWindowTitle("Open a new fit config file")
-            filename.setModal(1)
-            filename.setFileMode(filename.ExistingFiles)
-            filename.setDirectory(wdir)
-        filetypes = "Config Files (*.cfg)\nAll files (*)"
-        if PyMcaDirs.nativeFileDialogs:
-            try:
-                # API 1
-                filenameList = qt.QFileDialog.getOpenFileNames(self,
-                                "Open a new fit config file",
-                                wdir,
-                                filetypes,
-                                None)    #This should be the filter used
-            except:
-                # API 2
-                filenameList, andFilter = \
-                              qt.QFileDialog.getOpenFileNamesAndFilter(self,
-                                "Open a new fit config file",
-                                wdir,
-                                filetypes,
-                                None)    #This should be the filter used
-        else:
-            if hasattr(filename, "setFilters"):
-                filename.setFilters(["Config Files (*.cfg)",
-                                     "All files (*)"])
-            else:
-                filename.setNameFilters(["Config Files (*.cfg)",
-                                         "All files (*)"])
-            ret = filename.exec_()
+        filetypelist = ["Config Files (*.cfg)", "All files (*)"]
+        fileList, filefilter = PyMcaFileDialogs.getFileList(self,
+                                                filetypelist=filetypelist,
+                                                message="Open a new fit config file",
+                                                currentdir=wdir,
+                                                mode="OPEN",
+                                                getfilter=True,
+                                                single=False,
+                                                currentfilter=None)
 
-            if  ret == qt.QDialog.Accepted:
-                filenameList = filename.selectedFiles()
-            else:
-                self.raise_()
-                return
-
-        filename = []
-        for f in filenameList:
-            filename.append(qt.safe_str(f))
-
-        if len(filename) == 1:
-            self.setConfigFile(qt.safe_str(filename[0]))
-        elif len(filenameList):
-            self.setConfigFile(filename)
+        if len(fileList) == 1:
+            self.setConfigFile(fileList[0])
+        elif len(fileList):
+            self.setConfigFile(fileList)
         if QTVERSION < '4.0.0':
             self.raiseW()
         else:
@@ -738,29 +682,12 @@ class McaBatchGUI(qt.QWidget):
         if not os.path.exists(self.outputDir):
             self.outputDir =  os.getcwd()
         wdir = self.outputDir
-        if QTVERSION < '4.0.0':
-            outfile = qt.QFileDialog(self,"Output Directory Selection",1)
-            outfile.setMode(outfile.DirectoryOnly)
-            outfile.setDir(wdir)
-            ret = outfile.exec_loop()
-        else:
-            outfile = qt.QFileDialog(self)
-            outfile.setWindowTitle("Output Directory Selection")
-            outfile.setModal(1)
-            outfile.setDirectory(wdir)
-            outfile.setFileMode(outfile.DirectoryOnly)
-            ret = outfile.exec_()
-        if ret:
-            if QTVERSION < '4.0.0':
-                outdir=qt.safe_str(outfile.selectedFile())
-            else:
-                outdir=qt.safe_str(outfile.selectedFiles()[0])
-            outfile.close()
-            del outfile
+        outdir =PyMcaFileDialogs.getExistingDirectory(self,
+                                    message="Output Directory Selection",
+                                    mode="SAVE",
+                                    currentdir=wdir)
+        if len(outdir):
             self.setOutputDir(outdir)
-        else:
-            outfile.close()
-            del outfile
         if QTVERSION < '4.0.0':
             self.raiseW()
         else:
@@ -785,11 +712,9 @@ class McaBatchGUI(qt.QWidget):
                     allowSingleFileSplitProcesses = False
                     if HDF5SUPPORT:
                         if h5py.is_hdf5(self.fileList[0]):
-                            if DEBUG:
-                                print("Disallow single HDF5 process split")
-                                print("due to problems with concurrent access")
-                            #allowSingleFileSplitProcesses = True
-                            allowSingleFileSplitProcesses = False
+                            _logger.info("Allowing single HDF5 file process split")
+                            _logger.info("In the past it was problematic")
+                            allowSingleFileSplitProcesses = True
                     if not allowSingleFileSplitProcesses:
                         text = "Multiple processes can only be used with multiple input files."
                         qt.QMessageBox.critical(self, "ERROR",text)
@@ -1006,8 +931,7 @@ class McaBatchGUI(qt.QWidget):
             self.hide()
             qApp = qt.QApplication.instance()
             qApp.processEvents()
-            if DEBUG:
-                print("cmd = %s" % cmd)
+            _logger.debug("cmd = %s", cmd)
             if self.__splitBox.isChecked():
                 nbatches = int(qt.safe_str(self.__splitSpin.text()))
                 if len(self.fileList) > 1:
@@ -1027,8 +951,7 @@ class McaBatchGUI(qt.QWidget):
                                 subprocess.Popen(cmd1.encode(sys.getfilesystemencoding()),
                                                  cwd=os.getcwd()))
 
-                        if DEBUG:
-                            print("cmd = %s" % cmd1)
+                        _logger.debug("cmd = %s", cmd1)
                 else:
                     #f = open("CMD", 'wb')
                     processList = []
@@ -1036,8 +959,7 @@ class McaBatchGUI(qt.QWidget):
                         #the mcastep has been dealt with above
                         cmd1        = cmd + " --mcaoffset=%d --chunk=%d" % (i, i)
                         processList.append(subprocess.Popen(cmd1, cwd=os.getcwd()))
-                        if DEBUG:
-                            print("CMD = %s" % cmd1)
+                        _logger.debug("CMD = %s", cmd1)
                         #f.write(cmd1+"\n")
                     #f.close()
                 self._processList = processList
@@ -1047,7 +969,7 @@ class McaBatchGUI(qt.QWidget):
                 if not self._timer.isActive():
                     self._timer.start(1000)
                 else:
-                    print("timer was already active")
+                    _logger.info("timer was already active")
                 return
             else:
                 try:
@@ -1134,8 +1056,7 @@ class McaBatchGUI(qt.QWidget):
                                                     self.outputDir, overwrite,
                                                     filestep, mcastep, html, htmlindex,
                                                     listfile, concentrations, table, fitfiles, selectionFlag)
-            if DEBUG:
-                print("cmd = %s" % cmd)
+            _logger.debug("cmd = %s", cmd)
             if self.__splitBox.isChecked():
                 qApp = qt.QApplication.instance()
                 qApp.processEvents()
@@ -1167,11 +1088,11 @@ class McaBatchGUI(qt.QWidget):
                 if not self._timer.isActive():
                     self._timer.start(1000)
                 else:
-                    print("timer was already active")
+                    _logger.info("timer was already active")
                 return
             else:
                 os.system(cmd)
-                print(" COMMAND = ", cmd)
+                _logger.info(" COMMAND = %s", cmd)
                 msg = qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Information)
                 text = "Your batch has been started as an independent process."
@@ -1183,7 +1104,7 @@ class McaBatchGUI(qt.QWidget):
             try:
                 os.remove(listfile)
             except:
-                print("Cannot delete file %s" % listfile)
+                _logger.error("Cannot delete file %s", listfile)
                 raise
         if config is None:
             lst = self.fileList
@@ -1223,7 +1144,7 @@ class McaBatchGUI(qt.QWidget):
             self.raise_()
 
         work = PyMcaBatchBuildOutput.PyMcaBatchBuildOutput(os.path.join(self.outputDir, "IMAGES"))
-        if DEBUG:
+        if _logger.getEffectiveLevel() == logging.DEBUG:
             a, b, c = work.buildOutput(delete=False)
         else:
             a, b, c = work.buildOutput(delete=True)
@@ -1244,7 +1165,7 @@ class McaBatchGUI(qt.QWidget):
                 else:
                     os.system("%s %s &" % (rgb, b[0]))
         work = PyMcaBatchBuildOutput.PyMcaBatchBuildOutput(self.outputDir)
-        if DEBUG:
+        if _logger.getEffectiveLevel() == logging.DEBUG:
             work.buildOutput(delete=False)
         else:
             work.buildOutput(delete=True)
@@ -1300,8 +1221,7 @@ class McaBatch(McaAdvancedFitBatch.McaAdvancedFitBatch, qt.QThread):
             qt.QApplication.postEvent(self.parent, McaCustomEvent.McaCustomEvent(ddict))
 
     def onMca(self, mca, nmca, filename=None, key=None, info=None):
-        if DEBUG:
-            print("onMca key = %s" % key)
+        _logger.debug("onMca key = %s", key)
         ddict = {'mca':mca,
                  'nmca':nmca,
                  'mcastep':self.mcaStep,
@@ -1321,8 +1241,7 @@ class McaBatch(McaAdvancedFitBatch.McaAdvancedFitBatch, qt.QThread):
         if self.pleasePause:self.__pauseMethod()
 
     def onEnd(self):
-        if DEBUG:
-            print("onEnd")
+        _logger.debug("onEnd")
 
         ddict = {'event':'onEnd',
                  'filestep':self.fileStep,
@@ -1453,12 +1372,11 @@ class McaBatchWindow(qt.QWidget):
         elif event.dict['event'] == 'reportWritten':self.onReportWritten()
 
         else:
-            print("Unhandled event %s" % event)
+            _logger.warning("Unhandled event %s", event)
 
 
     def onNewFile(self, file, filelist, filestep, filebeginoffset =0, fileendoffset = 0):
-        if DEBUG:
-            print("onNewFile: %s" % file)
+        _logger.debug("onNewFile: %s", file)
         indexlist = list(range(0,len(filelist),filestep))
         index  = indexlist.index(filelist.index(file)) - filebeginoffset
         #print index + filebeginoffset
@@ -1473,7 +1391,7 @@ class McaBatchWindow(qt.QWidget):
                     try:
                         os.remove(self.htmlindex)
                     except:
-                        print("cannot delete file %s" % self.htmlindex)
+                        _logger.warning("cannot delete file %s", self.htmlindex)
         nfiles = len(indexlist)-filebeginoffset-fileendoffset
         self.status.setText("Processing file %s" % file)
         e = time.time()
@@ -1496,9 +1414,8 @@ class McaBatchWindow(qt.QWidget):
             qApp = qt.QApplication.instance()
             qApp.processEvents()
 
-    def onImage(self,key,keylist):
-        if DEBUG:
-            print("onImage %s" % key)
+    def onImage(self, key, keylist):
+        _logger.debug("onImage %s",  key)
         i = keylist.index(key) + 1
         n = len(keylist)
         if QTVERSION < '4.0.0':
@@ -1515,8 +1432,7 @@ class McaBatchWindow(qt.QWidget):
 
     #def onMca(self, mca, nmca, mcastep):
     def onMca(self, ddict):
-        if DEBUG:
-            print("onMca ", ddict['mca'])
+        _logger.debug("onMca %s", ddict['mca'])
         mca  = ddict['mca']
         nmca = ddict['nmca']
         mcastep  = ddict['mcastep']
@@ -1536,12 +1452,12 @@ class McaBatchWindow(qt.QWidget):
                         self.__htmlReport(filename, key, outputdir,
                                           useExistingFiles, info, firstmca = False)
             except:
-                print("ERROR on REPORT",sys.exc_info())
-                print(sys.exc_info()[1])
-                print("filename = %s key =%s " % (filename, key))
-                print("If your batch is stopped, please report this")
-                print("error sending the above mentioned file and the")
-                print("associated fit configuration file.")
+                _logger.warning("ERROR on REPORT %s", sys.exc_info())
+                _logger.warning("%s", sys.exc_info()[1])
+                _logger.warning("filename = %s key =%s ", (filename, key))
+                _logger.warning("If your batch is stopped, please report this")
+                _logger.warning("error sending the above mentioned file and the")
+                _logger.warning("associated fit configuration file.")
         if QTVERSION < '4.0.0':
             self.mcaBar.setTotalSteps(nmca)
             self.mcaBar.setProgress(mca)
@@ -1565,20 +1481,20 @@ class McaBatchWindow(qt.QWidget):
             try:
                 os.mkdir(fitdir)
             except:
-                print("I could not create directory %s" % fitdir)
+                _logger.warning("I could not create directory %s", fitdir)
                 return
-        fitdir = os.path.join(fitdir,filename+"_HTMLDIR")
+        fitdir = os.path.join(fitdir, filename+"_HTMLDIR")
         if not os.path.exists(fitdir):
             try:
                 os.mkdir(fitdir)
             except:
-                print("I could not create directory %s" % fitdir)
+                _logger.warning("I could not create directory %s", fitdir)
                 return
         localindex = os.path.join(fitdir, "index.html")
         if not os.path.isdir(fitdir):
-            print("%s does not seem to be a valid directory" % fitdir)
+            _logger.warning("%s does not seem to be a valid directory", fitdir)
         else:
-            outfile = filename +"_"+key+".html"
+            outfile = filename + "_" + key + ".html"
             outfile = os.path.join(fitdir,  outfile)
         useExistingResult = useExistingFiles
         if os.path.exists(outfile):
@@ -1586,7 +1502,7 @@ class McaBatchWindow(qt.QWidget):
                 try:
                     os.remove(outfile)
                 except:
-                    print("cannot delete file %s" % outfile)
+                    _logger.warning("cannot delete file %s", outfile)
                 useExistingResult = 0
         else:
             useExistingResult = 0
@@ -1595,7 +1511,7 @@ class McaBatchWindow(qt.QWidget):
         fitdir = os.path.join(fitdir,filename+"_FITDIR")
         fitfile= os.path.join(fitdir,  filename +"_"+key+".fit")
         if not os.path.exists(fitfile):
-            print("fit file %s does not exists!" % fitfile)
+            _logger.warning("fit file %s does not exists!", fitfile)
             return
         if self.report is None:
             #first file
@@ -1703,8 +1619,7 @@ class McaBatchWindow(qt.QWidget):
                 if sys.executable in ["PyMcaMain", "PyMcaMain.exe",
                                       "PyMcaBatch", "PyMcaBatch.exe"]:
                     frozen = True
-            if DEBUG:
-                print("final dirname = %s" % dirname)
+            _logger.debug("final dirname = %s", dirname)
             if frozen:
                 # we are at level PyMca5\PyMcaGui\pymca
                 dirname  = os.path.dirname(dirname)
@@ -1717,19 +1632,20 @@ class McaBatchWindow(qt.QWidget):
             else:
                 myself  = sys.executable+" "+os.path.join(dirname, "EdfFileSimpleViewer.py")
             cmd = "%s %s &" % (myself, filelist)
-            if DEBUG:
-                print("cmd = %s" % cmd)
+            _logger.debug("cmd = %s", cmd)
             os.system(cmd)
 
 def main():
     sys.excepthook = qt.exceptionHandler
     import getopt
+    from PyMca5.PyMcaCore.LoggingLevel import getLoggingLevel
     options     = 'f'
     longoptions = ['cfg=','outdir=','roifit=','roi=','roiwidth=',
                    'overwrite=', 'filestep=', 'mcastep=', 'html=','htmlindex=',
                    'listfile=','cfglistfile=', 'concentrations=', 'table=', 'fitfiles=',
                    'filebeginoffset=','fileendoffset=','mcaoffset=', 'chunk=',
-                   'nativefiledialogs=','selection=', 'exitonend=']
+                   'nativefiledialogs=','selection=', 'exitonend=',
+                   'logging=', 'debug=']
     filelist = None
     outdir   = None
     cfg      = None
@@ -1806,6 +1722,8 @@ def main():
         elif opt in ('--exitonend'):
             exitonend = int(arg)
 
+    logging.basicConfig(level=getLoggingLevel(opts))
+
     if listfile is None:
         filelist=[]
         for item in args:
@@ -1858,8 +1776,8 @@ def main():
                       mcaoffset=mcaoffset, chunk=chunk, selection=selection)
         except:
             if exitonend:
-                print("Error: " % sys.exc_info()[1])
-                print("Quitting as requested")
+                _logger.warning("Error: ", sys.exc_info()[1])
+                _logger.warning("Quitting as requested")
                 qt.QApplication.instance().quit()
             else:
                 msg = qt.QMessageBox()
