@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2015 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -29,8 +29,7 @@ __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import numpy
-import time
-from PyMca5.PyMcaGui import IconDict
+import logging
 from . import RGBImageCalculator
 qt = RGBImageCalculator.qt
 QTVERSION = qt.qVersion()
@@ -38,7 +37,8 @@ from . import RGBCorrelator
 from PyMca5.PyMcaGui import FrameBrowser
 USE_BROWSER = True
 
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 class PyMcaImageWindow(RGBImageCalculator.RGBImageCalculator):
     def __init__(self, parent = None,
@@ -99,8 +99,7 @@ class PyMcaImageWindow(RGBImageCalculator.RGBImageCalculator):
         w.sigReplaceSelection.connect(self._replaceSelection)
 
     def _addSelection(self, selectionlist):
-        if DEBUG:
-            print("_addSelection(self, selectionlist)",selectionlist)
+        _logger.debug("_addSelection(self, selectionlist=%s)", selectionlist)
         if type(selectionlist) == type([]):
             sellist = selectionlist
         else:
@@ -121,6 +120,7 @@ class PyMcaImageWindow(RGBImageCalculator.RGBImageCalculator):
             if dataObject.info.get("selectiontype", "1D") != "2D":
                 continue
             if dataObject.data is None:
+                # This is the SCAN regular mesh case
                 if hasattr(dataObject, "y"):
                     if dataObject.y is not None:
                         dataObject.data = dataObject.y[0]
@@ -160,10 +160,46 @@ class PyMcaImageWindow(RGBImageCalculator.RGBImageCalculator):
                                             tmpData = numpy.transpose(dataObject.y[yIndex])[:]
                                         else:
                                             tmpData = dataObject.y[yIndex][:]
-                                        tmpData.shape = nRows, nColumns
+                                        tmpData.shape = int(nRows), int(nColumns)
                                         dataObject.data[yIndex] = tmpData
                     else:
-                        print("Nothing to plot")
+                        _logger.info("Nothing to plot")
+            elif hasattr(dataObject, "x") and (dataObject.x is not None):
+                shape = dataObject.data.shape
+                if len(dataObject.x) == 2:
+                    x0 = dataObject.x[0][:]
+                    x0.shape = -1
+                    x1 = dataObject.x[1][:]
+                    x1.shape = -1
+                    if abs(x0[1] - x0[0]) < 1.0e-6:
+                        nColumns = numpy.argmin(abs(x0-x0[0]) < 1.0e-6)
+                        nRows = x1.size / nColumns
+                        if nRows!= int(nRows):
+                            _logger.warning("%f != %d", nRows, int(nRows))
+                            raise ValueError("2D Selection not understood")
+                        transpose = False
+                        nColumns = int(nColumns)
+                        self._yScale = x0[0], x0[nColumns] - x0[0]
+                        self._xScale = x1[0], x1[1] - x1[0]
+                    elif abs(x1[1] - x1[0]) < 1.0e-6:
+                        nRows = numpy.argmin(abs(x1-x1[0]) < 1.0e-6)
+                        nColumns = x0.size / nRows
+                        if nColumns != int(nColumns):
+                            _logger.warning("%f != %d", nColumns, int(nColumns))
+                            raise ValueError("2D Selection not understood")
+                        transpose = True
+                        nRows = int(nRows)
+                        self._xScale = x0[0], x0[1] - x0[0]
+                        self._yScale = x1[0], x1[nRows] - x1[0]
+                    elif (len(x0) == shape[-2]) and (len(x1) == shape[-1]):
+                        self._xScale = x1[0], x1[1] - x1[0]
+                        self._yScale = x0[0], x0[1] - x0[0]
+                    elif (len(x0) == shape[-1]) and (len(x1) == shape[-2]):
+                        self._yScale = x1[0], x1[1] - x1[0]
+                        self._xScale = x0[0], x0[1] - x0[0]
+                    else:
+                        raise TypeError("2D Selection is not a regular mesh")
+
             self.dataObjectsList = [legend]
             self.dataObjectsDict = {legend:dataObject}
             shape = dataObject.data.shape
@@ -262,8 +298,7 @@ class PyMcaImageWindow(RGBImageCalculator.RGBImageCalculator):
                 pass
 
     def _removeSelection(self, selectionlist):
-        if DEBUG:
-            print("_removeSelection(self, selectionlist)",selectionlist)
+        _logger.debug("_removeSelection(self, selectionlist=%s)", selectionlist)
         if type(selectionlist) == type([]):
             sellist = selectionlist
         else:
@@ -279,8 +314,8 @@ class PyMcaImageWindow(RGBImageCalculator.RGBImageCalculator):
                 #self.plotImage(True)
 
     def _replaceSelection(self, selectionlist):
-        if DEBUG:
-            print("_replaceSelection(self, selectionlist)",selectionlist)
+        _logger.debug("_replaceSelection(self, selectionlist=%s)",
+                      selectionlist)
         current = self.slider.value()
         self._addSelection(selectionlist)
         if current < self._nImages:
@@ -325,7 +360,7 @@ class TimerLoop:
         self.__timer.start(period)
 
     def test(self):
-        print("Test function called")
+        _logger.info("Test function called")
 
 if __name__ == "__main__":
     from PyMca5 import DataObject
@@ -341,8 +376,7 @@ if __name__ == "__main__":
     def buildSelection(dataObject, name = "image_data0"):
         key = dataObject.info['Key']
         def dataObjectDestroyed(ref, dataObjectKey=key):
-            if DEBUG:
-                print("dataObject distroyed key = %s" % key)
+            _logger.debug("dataObject distroyed key = %s", key)
         dataObjectRef=weakref.proxy(dataObject, dataObjectDestroyed)
         selection = {}
         selection['SourceType'] = 'SPS'
