@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2016 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -30,11 +30,12 @@ __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys, getopt
 import traceback
+import logging
 if sys.platform == 'win32':
     import ctypes
     from ctypes.wintypes import MAX_PATH
 nativeFileDialogs = None
-DEBUG = 0
+_logger = logging.getLogger(__name__)
 backend=None
 if __name__ == '__main__':
     options     = '-f'
@@ -44,27 +45,32 @@ if __name__ == '__main__':
                    'qt=',
                    'backend=',
                    'nativefiledialogs=',
-                   'PySide=']
+                   'PySide=',
+                   'binding=',
+                   'logging=']
     try:
         opts, args = getopt.getopt(
                      sys.argv[1:],
                      options,
                      longoptions)
     except getopt.error:
-        print(sys.exc_info()[1])
+        print("%s" % sys.exc_info()[1])
         sys.exit(1)
 
     keywords={}
     debugreport = 0
-    qtversion = '4'
+    qtversion = None
+    binding = None
     for opt, arg in opts:
         if  opt in ('--spec'):
             keywords['spec'] = arg
         elif opt in ('--shm'):
             keywords['shm']  = arg
         elif opt in ('--debug'):
-            debugreport = 1
-            DEBUG = 1
+            if arg.lower() not in ['0', 'false']:
+                debugreport = 1
+                _logger.setLevel(logging.DEBUG)
+            # --debug is also parsed later for the global logging level
         elif opt in ('-f'):
             keywords['fresh'] = 1
         elif opt in ('--qt'):
@@ -77,20 +83,76 @@ if __name__ == '__main__':
             else:
                 nativeFileDialogs = False
         elif opt in ('--PySide'):
-            import PySide
-    if qtversion == '3':
-        raise NotImplementedError("Qt3 is not longer supported")
+            print("Please use --binding=PySide")
+            import PySide.QtCore
+        elif opt in ('--binding'):
+            binding = arg.lower()
+            if binding == "pyqt5":
+                import PyQt5.QtCore
+            elif binding == "pyqt4":
+                if sys.version_info < (3,):
+                    try:
+                        import sip
+                        sip.setapi('QString', 2)
+                        sip.setapi('QVariant', 2)
+                        sip.setapi('QDate', 2)
+                        sip.setapi('QDateTime', 2)
+                        sip.setapi('QTextStream', 2)
+                        sip.setapi('QTime', 2)
+                        sip.setapi('QUrl', 2)
+                    except:
+                        print("Cannot set sip API")
+                import PyQt4.QtCore
+            elif binding == "pyside2":
+                import PySide2.QtCore
+            elif binding == "pyside":
+                import PySide.QtCore
+            else:
+                raise ValueError("Unknown Qt binding <%s>" % binding)
+    if binding is None:
+        if qtversion == '3':
+            raise NotImplementedError("Qt3 is no longer supported")
+        elif qtversion == '4':
+            if sys.version_info < (3,):
+                try:
+                    import sip
+                    sip.setapi("QString", 2)
+                    sip.setapi("QVariant", 2)
+                except:
+                    print("Cannot set sip API")
+            import PyQt4.QtCore
+        elif qtversion == '5':
+            import PyQt5.QtCore
+
+    from PyMca5.PyMcaCore.LoggingLevel import getLoggingLevel
+    logging.basicConfig(level=getLoggingLevel(opts))
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
+QTVERSION = qt.qVersion()
+if sys.platform == 'darwin':
+    if backend is not None:
+        if backend.lower() in ["gl", "opengl"]:
+            if hasattr(qt, 'QOpenGLWidget'):
+                print("Warning: OpenGL backend not fully supported")
+try:
+    if (sys.version_info < (3,)) and ("PyQt4.QtCore" in sys.modules):
+        # PyQt4 is the most reliable binding at the ESRF for python 2
+        import silx
+        silxLogger = logging.getLogger("silx.DEPRECATION")
+        silxLogger.setLevel(logging.CRITICAL)
+    # try to import silx prior to importing matplotlib to prevent
+    # unnecessary warning
+    import silx.gui.plot
+except:
+    pass
 from PyMca5.PyMcaGui.pymca import PyMcaMdi
 IconDict = PyMcaMdi.IconDict
+IconDict0 = PyMcaMdi.IconDict0
 if hasattr(qt, "QString"):
     QString = qt.QString
 else:
     QString = qt.safe_str
 
-QTVERSION = qt.qVersion()
-
 try:
     from PyMca5.PyMcaGui.physics.xrf import XRFMCPyMca
     XRFMC_FLAG = True
@@ -103,8 +165,10 @@ try:
 except:
     SUMRULES_FLAG = False
 
+# prefer lazy import to avoid OpenCL related crashes on startup
+TOMOGUI_FLAG = True
+
 import PyMca5
-from PyMca5.PyMcaGui.pymca.PyMca_help import HelpDict
 from PyMca5 import PyMcaDataDir
 import os
 __version__ = PyMca5.version()
@@ -166,7 +230,6 @@ if __name__ == "__main__":
         qApp = qt.QApplication.instance()
         qApp.processEvents()
 
-from PyMca5.PyMcaGraph.Plot import Plot
 from PyMca5.PyMcaGui.pymca import ScanWindow
 from PyMca5.PyMcaGui.pymca import McaWindow
 
@@ -241,13 +304,14 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.__imagingTool = None
             self._xrfmcTool = None
             self._sumRulesTool = None
+            self.__reconsWidget = None
             self.openMenu = qt.QMenu()
             self.openMenu.addAction("PyMca Configuration", self.openSource)
             self.openMenu.addAction("Data Source",
                          self.sourceWidget.sourceSelector._openFileSlot)
             self.openMenu.addAction("Load Training Data",
                                         self.loadTrainingData)
-
+            self.trainingDataMenu = None
 
             self.__useTabWidget = True
 
@@ -261,13 +325,14 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 self.mdi.addWindow(self.scanWindow)
             else:
                 if backend is not None:
-                    Plot.defaultBackend = backend
+                    import silx
+                    silx.config.DEFAULT_PLOT_BACKEND = backend
                 self.mainTabWidget = qt.QTabWidget(self.mdi)
                 self.mainTabWidget.setWindowTitle("Main Window")
                 self.mcaWindow = McaWindow.McaWindow(backend=backend)
                 self.scanWindow = ScanWindow.ScanWindow(info=True,
                                                         backend=backend)
-                self.scanWindow._togglePointsSignal()
+                self.scanWindow.getCurveStyleAction().trigger()
                 if OBJECT3D:
                     self.glWindow = SceneGLWindow.SceneGLWindow()
                 self.mainTabWidget.addTab(self.mcaWindow, "MCA")
@@ -309,7 +374,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     self.mcaWindow.showMaximized()
             currentConfigDict = ConfigDict.ConfigDict()
             try:
-                defaultFileName = self.__getDefaultSettingsFile()
+                defaultFileName = PyMca5.getDefaultSettingsFile()
                 self.configDir  = os.path.dirname(defaultFileName)
             except:
                 if not ('fresh' in kw):
@@ -333,7 +398,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                      self._startupSelection(source=kw['spec'],
                                                 selection=None)
 
-    def connectDispatcher(self, viewer, dispatcher = None):
+    def connectDispatcher(self, viewer, dispatcher=None):
         #I could connect sourceWidget to myself and then
         #pass the selections to the active window!!
         #That will be made in a next iteration I guess
@@ -400,22 +465,22 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             if self.mainTabWidget.isHidden():
                 #make sure it is visible in case of being closed
                 self.mainTabWidget.show()
-        if DEBUG:
+
+        try:
             self._dispatcherAddSelectionSlot(ddict)
-        else:
-            try:
-                self._dispatcherAddSelectionSlot(ddict)
-            except:
-                msg = qt.QMessageBox(self)
-                msg.setIcon(qt.QMessageBox.Critical)
-                msg.setText("Error: %s" % sys.exc_info()[1])
-                msg.setInformativeText(str(sys.exc_info()[1]))
-                msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+        except:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                raise
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Critical)
+            msg.setText("Error: %s" % sys.exc_info()[1])
+            msg.setInformativeText(str(sys.exc_info()[1]))
+            msg.setDetailedText(traceback.format_exc())
+            msg.exec_()
 
     def _dispatcherAddSelectionSlot(self, dictOrList):
-        if DEBUG:
-            print("self._dispatcherAddSelectionSlot(ddict), ddict = ", dictOrList)
+        _logger.debug("self._dispatcherAddSelectionSlot(ddict), ddict = %s",
+                      dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
@@ -423,8 +488,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
         toadd = False
         if self._is2DSelection(ddict):
-            if DEBUG:
-                print("2D selection")
+            _logger.debug("2D selection")
             if self.imageWindowCorrelator is None:
                 self.imageWindowCorrelator = RGBCorrelator.RGBCorrelator()
                 #toadd = True
@@ -446,13 +510,15 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 except:
                     pass
                 if hkl:
-                    imageWindow = PyMcaHKLImageWindow.PyMcaHKLImageWindow(name = legend,
-                                correlator = self.imageWindowCorrelator,
-                                scanwindow=self.scanWindow)
+                    imageWindow = PyMcaHKLImageWindow.PyMcaHKLImageWindow(
+                            name=legend,
+                            correlator=self.imageWindowCorrelator,
+                            scanwindow=self.scanWindow)
                 else:
-                    imageWindow = PyMcaImageWindow.PyMcaImageWindow(name = legend,
-                                correlator = self.imageWindowCorrelator,
-                                scanwindow=self.scanWindow)
+                    imageWindow = PyMcaImageWindow.PyMcaImageWindow(
+                            name=legend,
+                            correlator=self.imageWindowCorrelator,
+                            scanwindow=self.scanWindow)
                 self.imageWindowDict[legend] = imageWindow
 
                 imageWindow.sigAddImageClicked.connect( \
@@ -480,8 +546,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             else:
                 self.imageWindowDict[legend]._addSelection(ddict)
         elif self._isStackSelection(ddict):
-            if DEBUG:
-                print("Stack selection")
+            _logger.debug("Stack selection")
             legend = ddict['legend']
             widget = QStackWidget.QStackWidget()
             widget.notifyCloseEventToWidget(self)
@@ -492,18 +557,15 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         else:
             if OBJECT3D:
                 if ddict['dataobject'].info['selectiontype'] == "1D":
-                    if DEBUG:
-                        print("1D selection")
+                    _logger.debug("1D selection")
                     self.mcaWindow._addSelection(dictOrList)
                     self.scanWindow._addSelection(dictOrList)
                 else:
-                    if DEBUG:
-                        print("3D selection")
+                    _logger.debug("3D selection")
                     self.mainTabWidget.setCurrentWidget(self.glWindow)
                     self.glWindow._addSelection(dictOrList)
             else:
-                if DEBUG:
-                    print("1D selection")
+                _logger.debug("1D selection")
                 self.mcaWindow._addSelection(dictOrList)
                 self.scanWindow._addSelection(dictOrList)
 
@@ -518,8 +580,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
 
     def _dispatcherRemoveSelectionSlot(self, dictOrList):
-        if DEBUG:
-            print("self.dispatcherRemoveSelectionSlot(ddict), ddict = ",ddict)
+        _logger.debug("self.dispatcherRemoveSelectionSlot(ddict), ddict = %s", dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
@@ -550,8 +611,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             msg.exec_()
 
     def _dispatcherReplaceSelectionSlot(self, dictOrList):
-        if DEBUG:
-            print("self.dispatcherReplaceSelectionSlot(ddict), ddict = ", dictOrList)
+        _logger.debug("self.dispatcherReplaceSelectionSlot(ddict), ddict = %s",
+                      dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
@@ -585,8 +646,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.scanWindow._replaceSelection(dictOrList)
 
     def dispatcherOtherSignalsSlot(self, dictOrList):
-        if DEBUG:
-            print("self.dispatcherOtherSignalsSlot(ddict), ddict = ",dictOrList)
+        _logger.debug("self.dispatcherOtherSignalsSlot(ddict), ddict = %s",
+                      dictOrList)
         if type(dictOrList) == type([]):
             ddict = dictOrList[0]
         else:
@@ -605,8 +666,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         if ddict['event'] == "SourceTypeChanged":
             pass
             return
-        if DEBUG:
-            print("Unhandled dict")
+        _logger.debug("Unhandled dict")
 
     def setConfig(self, configDict):
         if 'PyMca' in configDict:
@@ -643,7 +703,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         d['PyMca']['Sources']['lastFileFilter'] = self.sourceWidget.sourceSelector.lastFileFilter
         for source in SOURCESLIST:
             d['PyMca'][source] = {}
-            if self.sourceWidget.sourceSelector.lastInputDir is not None:
+            if (self.sourceWidget.sourceSelector.lastInputDir is not None) and \
+               len(self.sourceWidget.sourceSelector.lastInputDir):
                 d['PyMca'][source]['lastInputDir'] = self.sourceWidget.sourceSelector.lastInputDir
                 try:
                     PyMcaDirs.inputDir = self.sourceWidget.sourceSelector.lastInputDir
@@ -681,12 +742,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         d["PyMca"]["McaWindow"]["calibrations"] = self.mcaWindow.getCalibrations()
 
         #ROIs
-        d['ROI']={}
-        if self.mcaWindow.roiWidget is None:
-            roilist = []
-            roidict = {}
-        else:
-            roilist, roidict = self.mcaWindow.roiWidget.getROIListAndDict()
+        d['ROI'] = {}
+        roilist, roidict = self.mcaWindow.getCurvesRoiDockWidget().roiWidget.getROIListAndDict()
         d['ROI']['roilist'] = roilist
         d['ROI']['roidict'] = {}
         d['ROI']['roidict'].update(roidict)
@@ -715,22 +772,21 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         #ScanFit related
         d['ScanSimpleFit'] = {}
         d['ScanSimpleFit']['Configuration'] = {}
-        if DEBUG:
-                  d['ScanSimpleFit']['Configuration'].update(\
-                      self.scanWindow.scanFit.getConfiguration())
-        else:
-            try:
-                  d['ScanSimpleFit']['Configuration'].update(\
-                      self.scanWindow.scanFit.getConfiguration())
-            except:
-                print("Error getting ScanFint configuration")
+
+        try:
+            d['ScanSimpleFit']['Configuration'].update(
+                self.scanWindow.scanFit.getConfiguration())
+        except:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                raise
+            _logger.warning("Error getting ScanFit configuration")
         return d
 
     def saveConfig(self, config, filename = None):
         d = ConfigDict.ConfigDict()
         d.update(config)
         if filename is None:
-            filename = self.__getDefaultSettingsFile()
+            filename = PyMca5.getDefaultSettingsFile()
         d.write(filename)
 
     def __configurePyMca(self, ddict):
@@ -744,6 +800,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             key = 'Splitter'
             if key in ddict['Geometry'].keys():
                 self.splitter.setSizes(ddict['Geometry'][key])
+            # TODO: Recover this functionality with silx
             if hasattr(self.mcaWindow, "graph"):
                 # this was the way of working of 4.x.x versions
                 key = 'McaWindow'
@@ -766,6 +823,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         native = ddict.get('nativeFileDialogs', True)
         if native in ["False", "0", 0]:
             native = False
+        else:
+            native = True
         PyMcaDirs.nativeFileDialogs = native
 
         if 'Sources' in ddict:
@@ -774,7 +833,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         for source in SOURCESLIST:
             if source in ddict:
                 if 'lastInputDir' in ddict[source]:
-                    if ddict[source] ['lastInputDir'] != "None":
+                    if ddict[source] ['lastInputDir'] not in ["None", []]:
                         self.sourceWidget.sourceSelector.lastInputDir =  ddict[source] ['lastInputDir']
                         try:
                             PyMcaDirs.inputDir = ddict[source] ['lastInputDir']
@@ -834,10 +893,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 if type(roilist) != type([]):
                     roilist=[roilist]
                 roidict = ddict['roidict']
-                if self.mcaWindow.roiWidget is None:
-                    self.mcaWindow.showRoiWidget(qt.Qt.BottomDockWidgetArea)
-                self.mcaWindow.roiWidget.fillFromROIDict(roilist=roilist,
-                                                         roidict=roidict)
+                # TODO: silx branch should show the ROI always with the McaWindow
+                roiWidget = self.mcaWindow.getCurvesRoiDockWidget().roiWidget
+                roiWidget.fillFromROIDict(roilist=roilist,
+                                          roidict=roidict)
 
     def __configureElements(self, ddict):
         if 'Material' in ddict:
@@ -880,8 +939,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                (d['LastFit']['ydata0'] != 'None'):
                 self.mcaWindow.advancedfit.setdata(x=d['LastFit']['xdata0'],
                                                    y=d['LastFit']['ydata0'],
-                                              sigmay=d['LastFit']['sigmay0'],
-                                              **d['Information'])
+                                                   sigmay=d['LastFit']['sigmay0'],
+                                                   **d['Information'])
                 if d['LastFit']['hidden'] == 'False':
                     self.mcaWindow.advancedfit.show()
                     self.mcaWindow.advancedfit.raiseW()
@@ -891,7 +950,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         except:
                             pass
                 else:
-                    print("hidden")
+                    _logger.info("hidden")
 
     def __configureScanCustomFit(self, ddict):
         pass
@@ -1002,7 +1061,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         layout1.addWidget(self.line1)
         #self.closelabel = qt.QLabel(sourceToolbar)
         self.closelabel = PixmapLabel(sourceToolbar)
-        self.closelabel.setPixmap(qt.QPixmap(IconDict['close']))
+        self.closelabel.setPixmap(qt.QPixmap(IconDict0['close']))
         layout1.addWidget(self.closelabel)
         self.closelabel.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed))
         #self.sourceBrowserTab=qt.QTabWidget(self.sourceFrame)
@@ -1065,8 +1124,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         """
 
     def menuToolsAboutToShow(self):
-        if DEBUG:
-            print("menu ToolsAboutToShow")
+        _logger.debug("menu ToolsAboutToShow")
         self.menuTools.clear()
         if self.sourceFrame.isHidden():
             self.menuTools.addAction("Show Source",self.toggleSource)
@@ -1089,8 +1147,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                                      self._xrfmcPyMca)
         if SUMRULES_FLAG:
             self.menuTools.addAction("Sum Rules Tool", self._sumRules)
-        if DEBUG:
-            print("Fit to Specfile missing")
+        _logger.debug("Fit to Specfile missing")
+        if TOMOGUI_FLAG:
+            self.menuTools.addAction("Tomography reconstruction",
+                                     self.__tomoRecons)
 
     def fontdialog(self):
         fontd = qt.QFontDialog.getFont(self)
@@ -1100,8 +1160,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
 
     def toggleSource(self,**kw):
-        if DEBUG:
-            print("toggleSource called")
+        _logger.debug("toggleSource called")
         if self.sourceFrame.isHidden():
             self.sourceFrame.show()
             self.sourceFrame.raise_()
@@ -1126,7 +1185,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         if self.identifier is None:
             self.identifier=PeakIdentifier.PeakIdentifier(energy=5.9,
                                 useviewer=1)
-            self.identifier.myslot()
+            self.identifier.mySlot()
         if self.identifier.isHidden():
             self.identifier.show()
         self.identifier.raise_()
@@ -1183,6 +1242,23 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         if self.__correlator is None:
             self.__correlator = []
 
+    def __tomoRecons(self):
+        if self.__reconsWidget is None:
+            try:
+                from tomogui.gui.ProjectWidget \
+                     import ProjectWindow as TomoguiProjectWindow
+            except ImportError:
+                msg = qt.QMessageBox(self)
+                msg.setIcon(qt.QMessageBox.Critical)
+                txt = "This functionality requires tomogui, silx and FreeART"
+                msg.setInformativeText(txt)
+                msg.setDetailedText(traceback.format_exc())
+                msg.exec_()
+                return
+            self.__reconsWidget = TomoguiProjectWindow()
+        if self.__reconsWidget.isHidden():
+            self.__reconsWidget.show()
+        self.__reconsWidget.raise_()
 
     def _deleteCorrelator(self, ddict):
         n = len(self.__correlator)
@@ -1314,6 +1390,20 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                             self.__imagingTool = None
                         del self._widgetDict[ddict['id']]
 
+    def closeEvent(self, event):
+        if __name__ == "__main__":
+            app = qt.QApplication.instance()
+            allWidgets = app.allWidgets()
+            for widget in allWidgets:
+                try:
+                    # we cannot afford to crash here
+                    if id(widget) != id(self):
+                        if widget.parent() is None:
+                            widget.close()
+                except:
+                    _logger.debug("Error closing widget")
+        return PyMcaMdi.PyMcaMdi.closeEvent(self, event)
+
     def __xiaCorrect(self):
         qApp = qt.QApplication.instance()
         XiaCorrect.mainGUI(qApp)
@@ -1345,8 +1435,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.saveMenu.addAction("Active Mca",
                              self.mcaWindow._saveIconSignal)
         elif text.upper() == 'SCAN':
-            self.saveMenu.addAction("Active Scan",
-                             self.scanWindow._saveIconSignal)
+            self.saveMenu.addAction(
+                    "Active Scan",
+                    self.scanWindow.getSaveAction().trigger)
+
         elif text in self.imageWindowDict.keys():
             self.saveMenu.addAction("Active Image",
                   self.imageWindowDict[text].graphWidget._saveIconSignal)
@@ -1416,52 +1508,31 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
     def _saveAs(self, filename=None):
         if filename is None:
-            filename = self.__getDefaultSettingsFile()
+            filename = PyMca5.getDefaultSettingsFile()
         self.saveConfig(self.getConfig(), filename)
 
-    def __getDefaultSettingsFile(self):
-        filename = "PyMca.ini"
-        if sys.platform == 'win32':
-            # recipe based on: http://bugs.python.org/issue1763#msg62242
-            dll = ctypes.windll.shell32
-            buf = ctypes.create_unicode_buffer(MAX_PATH + 1)
-            if dll.SHGetSpecialFolderPathW(None, buf, 0x0005, False):
-                directory = buf.value
-            else:
-                # the above should have worked
-                home = os.getenv('USERPROFILE')
-                try:
-                    l = len(home)
-                    directory = os.path.join(home, "My Documents")
-                except:
-                    home = '\\'
-                    directory = '\\'
-                #print home
-                #print directory
-            if os.path.isdir('%s' % directory):
-                directory = os.path.join(directory, "PyMca")
-            else:
-                #print "My Documents is not there"
-                directory = os.path.join(home, "PyMca")
-            if not os.path.exists('%s' % directory):
-                #print "PyMca directory not present"
-                os.mkdir('%s' % directory)
-            #print filename
-            finalfile = os.path.join(directory, filename)
-            #print finalfile
-        else:
-            home = os.getenv('HOME')
-            directory = os.path.join(home, "PyMca")
-            if not os.path.exists('%s' % directory):
-                os.mkdir('%s' % directory)
-            finalfile =  os.path.join(directory, filename)
-        return finalfile
-
     def loadTrainingData(self):
+        if self.trainingDataMenu is None:
+            self.trainingDataMenu = qt.QMenu()
+            self.trainingSources = {"XRF Analysis": os.path.join(PyMcaDataDir.PYMCA_DATA_DIR, 'XRFSpectrum.mca'),
+                        "Tertiary Excitation": os.path.join(PyMcaDataDir.PYMCA_DATA_DIR, 'Steel.spe')}
+            self.trainingActions = []
+            for key in ["XRF Analysis", "Tertiary Excitation"]:
+                action = qt.QAction(key, None)
+                self.trainingActions.append(action)
+                self.trainingDataMenu.addAction(action)
         try:
-            source = os.path.join(PyMcaDataDir.PYMCA_DATA_DIR,
-                                    'XRFSpectrum.mca')
+            selectedAction = self.trainingDataMenu.exec_(qt.QCursor.pos())
+            key = qt.safe_str(selectedAction.text())
+            source = self.trainingSources[key]
             self.sourceWidget.sourceSelector.openSource(source)
+            # only in case of the steel sample we set the input dir to simplify accessing the supplied cfg file
+            if key in ["Tertiary Excitation"]:
+                # we do not change the input dir currently used by the source selector
+                #self.sourceWidget.sourceSelector.lastInputDir = os.path.dirname(source)
+                # but we change the input dir to allow easy loading of the config file from the
+                # fit configuration window
+                PyMcaDirs.inputDir = os.path.dirname(source)
         except:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
@@ -1472,8 +1543,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             msg.exec_()
 
     def openSource(self,index=0):
-        if DEBUG:
-            print("index = %d " % index)
+        _logger.debug("index = %d ", index)
         if index <= 0:
             outfile = qt.QFileDialog(self)
             outfile.setWindowTitle("Select PyMca Configuration File")
@@ -1588,20 +1658,22 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         self.changeLog.show()
 
     def onDebug(self):
-        print("Module name PyQt  ",qt.PYQT_VERSION_STR)
+        _logger.debug("Module name PyQt  %s", qt.PYQT_VERSION_STR)
         for module in sys.modules.values():
             try:
                 if 'Revision' in module.__revision__:
                     if module.__name__ != "__main__":
-                        print("Module name = ",module.__name__,module.__revision__.replace("$",""))
+                        _logger.debug("Module name = %s, %s",
+                                      module.__name__,
+                                      module.__revision__.replace("$", ""))
             except:
                 pass
 
     def onPrint(self):
-        if DEBUG:
-            print("onPrint called")
+        _logger.debug("onPrint called")
+
         if not self.scanWindow.isHidden():
-            self.scanWindow.printGraph()
+            self.scanWindow.printPreview.menu().exec_(self.cursor().pos())
             return
 
         if not self.__useTabWidget:
@@ -1609,7 +1681,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.mcaWindow.raise_()
         else:
             self.mainTabWidget.setCurrentWidget(self.mcaWindow)
-        self.mcaWindow.printGraph()
+        self.mcaWindow.printPreview.menu().exec_(self.cursor().pos())
 
 if 0:
 
@@ -1624,7 +1696,7 @@ if 0:
 
             #name= self.__getNewGraphName()
             if name == "MCA Graph":
-                graph= McaWindow.McaWindow(self.mdi, name=name)
+                graph = McaWindow.McaWindow(self.mdi, name=name)
             graph.windowClosed[()].connect(self.closeGraph)
             graph.show()
 
@@ -1657,7 +1729,7 @@ if 0:
             """
             Called after a graph is closed
             """
-            print("closeGraph", name)
+            _logger.info("closeGraph", name)
 
     def __getGraphNames(self):
             return [ str(window.caption()) for window in self.mdi.windowList() ]
@@ -1697,8 +1769,7 @@ class MyQTextBrowser(qt.QTextBrowser):
 class Line(qt.QFrame):
     sigLineDoubleClickEvent = qt.pyqtSignal(object)
     def mouseDoubleClickEvent(self,event):
-        if DEBUG:
-            print("Double Click Event")
+        _logger.debug("Double Click Event")
         ddict={}
         ddict['event']="DoubleClick"
         ddict['data'] = event
@@ -1707,8 +1778,7 @@ class Line(qt.QFrame):
 class PixmapLabel(qt.QLabel):
     sigPixmapLabelMousePressEvent = qt.pyqtSignal(object)
     def mousePressEvent(self,event):
-        if DEBUG:
-            print("Mouse Press Event")
+        _logger.debug("Mouse Press Event")
         ddict={}
         ddict['event']="MousePress"
         ddict['data'] = event
