@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2016 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -33,12 +33,71 @@ import os
 import traceback
 import numpy
 import weakref
+import logging
+_logger = logging.getLogger(__name__)
+
+if __name__ == "__main__":
+    # we have to get the Qt binding prior to import PyMcaQt
+    import getopt
+    options = ''
+    longoptions = ["fileindex=","old",
+                   "filepattern=", "begin=", "end=", "increment=",
+                   "nativefiledialogs=", "imagestack=", "image=",
+                   "backend=", "binding=", "logging=", "debug="]
+    opts, args = getopt.getopt(
+                 sys.argv[1:],
+                 options,
+                 longoptions)
+    binding = None
+    for opt, arg in opts:
+        if opt in ('--debug'):
+            if arg.lower() not in ['0', 'false']:
+                debugreport = 1
+                _logger.setLevel(logging.DEBUG)
+            # --debug is also parsed later for the global logging level
+        elif opt in ('--binding'):
+            binding = arg.lower()
+            if binding == "pyqt5":
+                import PyQt5.QtCore
+            elif binding == "pyqt4":
+                if sys.version_info < (3,):
+                    try:
+                        import sip
+                        sip.setapi('QString', 2)
+                        sip.setapi('QVariant', 2)
+                        sip.setapi('QDate', 2)
+                        sip.setapi('QDateTime', 2)
+                        sip.setapi('QTextStream', 2)
+                        sip.setapi('QTime', 2)
+                        sip.setapi('QUrl', 2)
+                    except:
+                        print("Cannot set sip API")
+                import PyQt4.QtCore
+            elif binding == "pyside2":
+                import PySide2.QtCore
+            elif binding == "pyside":
+                import PySide.QtCore
+            else:
+                raise ValueError("Unknown Qt binding <%s>" % binding)
+    from PyMca5.PyMcaCore.LoggingLevel import getLoggingLevel
+    logging.basicConfig(level=getLoggingLevel(opts))
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
 if hasattr(qt, "QString"):
     QString = qt.QString
 else:
     QString = qt.safe_str
+try:
+    if (sys.version_info < (3,)) and ("PyQt4.QtCore" in sys.modules):
+        # PyQt4 is the most reliable binding at the ESRF for python 2
+        import silx
+        silxLogger = logging.getLogger("silx.DEPRECATION")
+        silxLogger.setLevel(logging.CRITICAL)
+    # try to import silx prior to importing matplotlib to prevent
+    # unnecessary warning
+    import silx.gui.plot
+except:
+    pass
 
 from PyMca5.PyMcaGui import PyMcaFileDialogs
 from PyMca5.PyMcaCore import DataObject
@@ -46,7 +105,8 @@ from PyMca5.PyMcaGui.pymca import McaWindow
 from PyMca5.PyMcaCore import StackBase
 from PyMca5.PyMcaGui import CloseEventNotifyingWidget
 from PyMca5.PyMcaGui import MaskImageWidget
-from PyMca5.PyMcaGui.pymca import StackROIWindow
+convertToRowAndColumn = MaskImageWidget.convertToRowAndColumn
+
 from PyMca5.PyMcaGui.pymca import RGBCorrelator
 from PyMca5.PyMcaGui.pymca.RGBCorrelatorWidget import ImageShapeDialog
 from PyMca5.PyMcaGui import IconDict
@@ -55,18 +115,19 @@ from PyMca5 import PyMcaDirs
 from PyMca5.PyMcaIO import ArraySave
 HDF5 = ArraySave.HDF5
 
-DEBUG = 0
+# _logger.setLevel(logging.DEBUG)
 QTVERSION = qt.qVersion()
-if DEBUG:
-    StackBase.DEBUG = DEBUG
+if _logger.getEffectiveLevel() == logging.DEBUG:
+    StackBase.logger.setLevel(logging.DEBUG)
+
 
 class QStackWidget(StackBase.StackBase,
                    CloseEventNotifyingWidget.CloseEventNotifyingWidget):
-    def __init__(self, parent = None,
-                 mcawidget = None,
-                 rgbwidget = None,
-                 vertical = False,
-                 master = True):
+    def __init__(self, parent=None,
+                 mcawidget=None,
+                 rgbwidget=None,
+                 vertical=False,
+                 master=True):
         StackBase.StackBase.__init__(self)
         CloseEventNotifyingWidget.CloseEventNotifyingWidget.__init__(self,
                                                                      parent)
@@ -89,7 +150,7 @@ class QStackWidget(StackBase.StackBase,
         self.mcaWidget = mcawidget
         self.rgbWidget = rgbwidget
         self.master = master
-        self._slave = None
+        self._slaveList = None
         self._masterStack = None
         self.stackSelector = None
         self._build(vertical=vertical)
@@ -160,16 +221,16 @@ class QStackWidget(StackBase.StackBase,
                                                          standalonesave=standaloneSaving,
                                                          profileselection=True,
                                                          aspect=True)
-        infotext  = 'Toggle background subtraction from current image\n'
+        infotext = 'Toggle background subtraction from current image\n'
         infotext += 'subtracting a straight line between the ROI limits.'
         self.roiBackgroundIcon = qt.QIcon(qt.QPixmap(IconDict["subtract"]))
-        self.roiBackgroundButton = self.roiWidget.graphWidget._addToolButton(\
+        self.roiBackgroundButton = self.roiWidget.graphWidget._addToolButton(
                                     self.roiBackgroundIcon,
                                     self._roiSubtractBackgroundClicked,
                                     infotext,
-                                    toggle = True,
-                                    state = False,
-                                    position = 6)
+                                    toggle=True,
+                                    state=False,
+                                    position=6)
         self.roiGraphWidget = self.roiWidget.graphWidget
         self.stackWindow.mainLayout.addWidget(self.stackWidget)
         self.roiWindow.mainLayout.addWidget(self.roiWidget)
@@ -182,37 +243,37 @@ class QStackWidget(StackBase.StackBase,
 
         #add some missing icons
         offset = 6
-        infotext  = 'If checked, spectra will be added normalized to the number\n'
+        infotext = 'If checked, spectra will be added normalized to the number\n'
         infotext += 'of pixels. Be carefull if you are preparing a batch and you\n'
         infotext += 'fit the normalized spectra because the data in the batch will\n'
         infotext += 'have a different weight because they are not normalized.'
         self.normalizeIcon = qt.QIcon(qt.QPixmap(IconDict["normalize16"]))
-        self.normalizeButton = self.stackGraphWidget._addToolButton(\
+        self.normalizeButton = self.stackGraphWidget._addToolButton( \
                                         self.normalizeIcon,
                                         self.normalizeIconChecked,
                                         infotext,
-                                        toggle = True,
-                                        state = False,
-                                        position = 6)
+                                        toggle=True,
+                                        state=False,
+                                        position=6)
         offset += 1
 
         if self.master:
             self.loadIcon = qt.QIcon(qt.QPixmap(IconDict["fileopen"]))
-            self.loadStackButton = self.stackGraphWidget._addToolButton(\
+            self.loadStackButton = self.stackGraphWidget._addToolButton( \
                                         self.loadIcon,
                                         self.loadSlaveStack,
                                         'Load another stack of same shape',
-                                        position = offset)
+                                        position=offset)
             offset += 1
 
-        self.pluginIcon     = qt.QIcon(qt.QPixmap(IconDict["plugin"]))
+        self.pluginIcon = qt.QIcon(qt.QPixmap(IconDict["plugin"]))
         infotext = "Call/Load Stack Plugins"
         self.stackGraphWidget._addToolButton(self.pluginIcon,
                                              self._pluginClicked,
                                              infotext,
-                                             toggle = False,
-                                             state = False,
-                                             position = offset)
+                                             toggle=False,
+                                             state=False,
+                                             position=offset)
 
     def setStack(self, *var, **kw):
         self.stackWidget.setImageData(None)
@@ -221,7 +282,7 @@ class QStackWidget(StackBase.StackBase,
         if (1 in self._stack.data.shape) and\
            isinstance(self._stack.data, numpy.ndarray):
             oldshape = self._stack.data.shape
-            dialog = ImageShapeDialog(self, shape = oldshape[0:2])
+            dialog = ImageShapeDialog(self, shape=oldshape[0:2])
             dialog.setModal(True)
             ret = dialog.exec_()
             if ret:
@@ -236,7 +297,7 @@ class QStackWidget(StackBase.StackBase,
             if 'SourceName' in self._stack.info:
                 if type(self._stack.info['SourceName']) == type([]):
                     if len(self._stack.info['SourceName']) == 1:
-                        title = qt.safe_str(self._stack.info['SourceName'])
+                        title = qt.safe_str(self._stack.info['SourceName'][0])
                     else:
                         f0 = qt.safe_str(self._stack.info['SourceName'][0])
                         f1 = qt.safe_str(self._stack.info['SourceName'][-1])
@@ -259,15 +320,21 @@ class QStackWidget(StackBase.StackBase,
     def _roiSubtractBackgroundClicked(self):
         if not len(self._ROIImageList):
             return
+        xScale = self._stack.info.get("xScale", None)
+        yScale = self._stack.info.get("yScale", None)
         if self.roiBackgroundButton.isChecked():
-            self.roiWidget.graphWidget.graph.setGraphTitle(\
-                                self._ROIImageNames[0]+" Net")
-            self.roiWidget.setImageData(self._ROIImageList[0]-\
-                                        self._ROIImageList[-1])
+            self.roiWidget.graphWidget.graph.setGraphTitle( \
+                                self._ROIImageNames[0] + " Net")
+            self.roiWidget.setImageData(self._ROIImageList[0] - \
+                                        self._ROIImageList[-1],
+                                        xScale=xScale,
+                                        yScale=yScale)
         else:
-            self.roiWidget.graphWidget.graph.setGraphTitle(\
+            self.roiWidget.graphWidget.graph.setGraphTitle( \
                                 self._ROIImageNames[0])
-            self.roiWidget.setImageData(self._ROIImageList[0])
+            self.roiWidget.setImageData(self._ROIImageList[0],
+                                        xScale=xScale,
+                                        yScale=yScale)
 
     def _stackSaveToolButtonSignal(self):
         self._stackSaveMenu.exec_(self.cursor().pos())
@@ -350,7 +417,7 @@ class QStackWidget(StackBase.StackBase,
 
         ArraySave.save3DArrayAsMonochromaticTiff(dataView,
                                     filename,
-                                    labels = None,
+                                    labels=None,
                                     dtype=dtype,
                                     mcaindex=mcaIndex)
 
@@ -360,8 +427,21 @@ class QStackWidget(StackBase.StackBase,
     def _getCroppedView(self):
         mcaIndex = self._stack.info.get('McaIndex', -1)
         #get limits
-        row0, row1 = self.stackWidget.graph.getGraphYLimits()
-        col0, col1 = self.stackWidget.graph.getGraphXLimits()
+        y0, y1 = self.stackWidget.graph.getGraphYLimits()
+        x0, x1 = self.stackWidget.graph.getGraphXLimits()
+        xScale = self._stack.info.get("xScale", None)
+        yScale = self._stack.info.get("yScale", None)
+        if mcaIndex in [0]:
+            shape = [self._stack.data.shape[1], self._stack.data.shape[2]]
+        elif mcaIndex in [1]:
+            shape = [self._stack.data.shape[0], self._stack.data.shape[2]]
+        else:
+            shape = [self._stack.data.shape[0], self._stack.data.shape[1]]
+        row0, col0 = convertToRowAndColumn( \
+                     x0, y0, shape, xScale=xScale, yScale=yScale, safe=True)
+        row1, col1 = convertToRowAndColumn( \
+                     x1, y1, shape, xScale=xScale, yScale=yScale, safe=True)
+
         #this should go to array save ...
         shape = self._stack.data.shape
         if mcaIndex in [0]:
@@ -381,7 +461,7 @@ class QStackWidget(StackBase.StackBase,
             row1 = int(min([row1+0.5, self._stack.data.shape[0]]))
             col0 = int(max([col0+0.5, 0]))
             col1 = int(min([col1+0.5, self._stack.data.shape[1]]))
-            view = self._stack.data[row0:row1+1, col0:col1+1,:]
+            view = self._stack.data[row0:row1+1, col0:col1+1, :]
         return view
 
     def saveStackAsNeXus(self, dtype=None, interpretation=None, compression=False):
@@ -467,14 +547,14 @@ class QStackWidget(StackBase.StackBase,
         if not len(filename):
             return
         ArraySave.save3DArrayAsHDF5(self._stack.data, filename,
-                                    labels = None, dtype=None, mode='nexus+')
+                                    labels=None, dtype=None, mode='nexus+')
 
     def saveStackAsSimpleHDF5(self):
         filename = self._getOutputHDF5Filename()
         if not len(filename):
             return
         ArraySave.save3DArrayAsHDF5(self._stack.data, filename,
-                                    labels = None, dtype=None, mode='simple')
+                                    labels=None, dtype=None, mode='simple')
 
     def saveStackAsSimplestHDF5(self):
         filename = self._getOutputHDF5Filename()
@@ -482,30 +562,38 @@ class QStackWidget(StackBase.StackBase,
             return
         view = self._getCroppedView()
         ArraySave.save3DArrayAsHDF5(view, filename,
-                                    labels = None, dtype=None, mode='simplest')
+                                    labels=None, dtype=None, mode='simplest')
 
     def loadStack(self):
         if self._stackImageData is not None:
             #clear with a small stack
             stack = DataObject.DataObject()
-            stack.data = numpy.zeros((100,100,100), numpy.float32)
+            stack.data = numpy.zeros((100, 100, 100), numpy.float32)
             self.setStack(stack)
-        if self.stackSelector  is None:
+        if self.stackSelector is None:
             self.stackSelector = StackSelector.StackSelector(self)
         stack = self.stackSelector.getStack()
-        if type(stack) == type([]):
+        if (type(stack) == type([])) or isinstance(stack, list):
             #aifira like, two stacks
             self.setStack(stack[0])
-            self._slave = None
-            slave = QStackWidget(master=False, rgbwidget=self.rgbWidget)
-            slave.setStack(stack[1])
-            self.setSlave(slave)
+            self._slaveList = None
+            if len(stack) > 1:
+                for i in range(1, len(stack)):
+                    if stack[i] is not None:
+                        slave = QStackWidget(master=False,
+                                             rgbwidget=self.rgbWidget)
+                        slave.setStack(stack[i])
+                        if slave is not None:
+                            if i == 1:
+                                self.setSlave(slave)
+                            else:
+                                self.addSlave(slave)
         else:
             self.setStack(stack)
 
     def loadSlaveStack(self):
-        if self._slave is not None:
-            actionList = ['Load Slave', 'Show Slave', 'Delete Slave']
+        if self._slaveList is not None:
+            actionList = ['Add Slave', 'Load Slaves', 'Show Slaves', 'Merge Slaves', 'Delete Slaves']
             menu = qt.QMenu(self)
             for action in actionList:
                 text = QString(action)
@@ -514,15 +602,59 @@ class QStackWidget(StackBase.StackBase,
             if a is None:
                 return None
             if qt.safe_str(a.text()).startswith("Load"):
-                self._slave = None
+                self._closeSlave()
             elif qt.safe_str(a.text()).startswith("Show"):
-                self._slave.show()
-                self._slave.raise_()
+                for slave in self._slaveList:
+                    slave.show()
+                    slave.raise_()
+                return
+            elif qt.safe_str(a.text()).startswith("Merge"):
+                masterStackDataObject = self.getStackDataObject()
+                try:
+                    # Use views to ensure no casting is done in case of different dtype
+                    # to save memory.
+                    # This is risky when the original stack is integers (overflow).
+                    for slave in self._slaveList:
+                        masterStackDataObject.data[:] = \
+                                            masterStackDataObject.data[:] + \
+                                            slave.getStackData()
+                except:
+                    msg = qt.QMessageBox(self)
+                    msg.setIcon(qt.QMessageBox.Critical)
+                    msg.setWindowTitle("Stack Summing Error")
+                    msg.setText("An error has occurred while summing the master and slave stacks")
+                    msg.setInformativeText(qt.safe_str(sys.exc_info()[1]))
+                    msg.setDetailedText(traceback.format_exc())
+                    msg.exec_()
+                if "McaLiveTime" in masterStackDataObject.info:
+                    try:
+                        for slave in self._slaveList:
+                            info = slave.getStackInfo()
+                            if "McaLiveTime" in info:
+                                info["McaLiveTime"].shape = \
+                                   masterStackDataObject.info["McaLiveTime"].shape                                
+                                masterStackDataObject.info["McaLiveTime"] += \
+                                        info["McaLiveTime"]
+                            else:
+                                raise ValueError("No compatible time information")
+                    except:
+                        msg = qt.QMessageBox(self)
+                        msg.setIcon(qt.QMessageBox.Critical)
+                        msg.setWindowTitle("Stack Time Summing Error")
+                        txt = "An error has occurred cumulating the master and slave times\n"
+                        txt += "Time information is lost"
+                        del masterStackDataObject.info["McaLiveTime"] 
+                        msg.setText(txt)
+                        msg.setInformativeText(qt.safe_str(sys.exc_info()[1]))
+                        msg.setDetailedText(traceback.format_exc())
+                        msg.exec_()
+                self._closeSlave()
+                self.setStack(masterStackDataObject)
                 return
             else:
-                self._slave = None
+                self._closeSlave()
                 return
-        if self.stackSelector  is None:
+        if self.stackSelector is None:
             self.stackSelector = StackSelector.StackSelector(self)
 
         try:
@@ -539,21 +671,47 @@ class QStackWidget(StackBase.StackBase,
             return
         if stack is None:
             return
-        if type(stack) == type([]):
-            stack = stack[0]
-
-        slave = QStackWidget(rgbwidget=self.rgbWidget,
-                                  master=False)
-        slave.setStack(stack)
-        self.setSlave(slave)
+        if (type(stack) == type([])) or (isinstance(stack, list)):
+            self._closeSlave()
+            for i in range(len(stack)):
+                if stack[i] is not None:
+                    slave = QStackWidget(master=False,
+                                         rgbwidget=widget.rgbWidget)
+                    slave.setStack(stack[i])
+                    widget.addSlave(slave)
+                    stack[i] = None
+        else:
+            slave = QStackWidget(rgbwidget=self.rgbWidget,
+                                 master=False)
+            slave.setStack(stack)
+            self.setSlave(slave)
 
+    def _closeSlave(self):
+        if self._slaveList is None:
+            return
+        for slave in self._slaveList:
+            slave.close()
+        slave = None
+        self._slaveList = None
+        # make sure memory is released
+        import gc
+        gc.collect()
 
     def setSlave(self, slave):
-        self._slave = None
-        self._slave = slave
-        self._slave.setSelectionMask(self.getSelectionMask())
-        self._slave.show()
-        self._slave._setMaster(self)
+        if self._slaveList is None:
+            self._slaveList = []
+        for slave in self._slaveList:
+            slave.close()
+        self._slaveList = None
+        self.addSlave(slave)
+
+    def addSlave(self, slave):
+        if self._slaveList is None:
+            self._slaveList = []
+        slave.setSelectionMask(self.getSelectionMask())
+        slave.show()
+        slave._setMaster(self)
+        self._slaveList.append(slave)
 
     def _setMaster(self, master=None):
         if self.master:
@@ -566,10 +724,13 @@ class QStackWidget(StackBase.StackBase,
     def getStackDataObjectList(self):
         stackList = []
         if self.master:
+            # master, join all slaves
             stackList.append(self.getStackDataObject())
-            if self._slave is not None:
-                stackList.append(self._slave.getStackDataObject())
+            if self._slaveList is not None:
+                for slave in self._slaveList:
+                    stackList.append(slave.getStackDataObject())
         else:
+            # slave, join master
             stackList.append(self._masterStack.getStackDataObject())
             stackList.append(self.getStackDataObject())
         return stackList
@@ -583,8 +744,8 @@ class QStackWidget(StackBase.StackBase,
         text = QString("Set User Plugin Directory")
         menu.addAction(text)
         actionList.append(text)
-        global DEBUG
-        if DEBUG:
+        global _logger
+        if _logger.getEffectiveLevel() == logging.DEBUG:
             text = QString("Toggle DEBUG mode OFF")
         else:
             text = QString("Toggle DEBUG mode ON")
@@ -643,11 +804,12 @@ class QStackWidget(StackBase.StackBase,
                 self.setPluginDirectoryList(pluginsDirList)
             return
         if idx == 2:
-            if DEBUG:
-                DEBUG = 0
+            if _logger.getEffectiveLevel() == logging.DEBUG:
+                _logger.setLevel(logging.DEBUG)
+                StackBase.logger.setLevel(logging.DEBUG)
             else:
-                DEBUG = 1
-            StackBase.DEBUG = DEBUG
+                _logger.setLevel(logging.NOTSET)
+                StackBase.logger.setLevel(logging.NOTSET)
             return
         key = callableKeys[idx]
         methods = self.pluginInstanceDict[key].getMethods()
@@ -690,7 +852,7 @@ class QStackWidget(StackBase.StackBase,
             msg.setInformativeText(qt.safe_str(sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
             msg.exec_()
-            if DEBUG:
+            if _logger.getEffectiveLevel() == logging.DEBUG:
                 raise
 
     def _actionHovered(self, action):
@@ -716,7 +878,7 @@ class QStackWidget(StackBase.StackBase,
                 self.mainLayout.addWidget(self.rgbWidget)
         elif n == 2:
             self.tab = qt.QTabWidget(self)
-            self.mcaWidget = McaWindow.McaWindow()#vertical=False)
+            self.mcaWidget = McaWindow.McaWindow() #vertical=False
             #self.mcaWidget.graph.setMinimumWidth(0.5 * \
             #                            qt.QWidget.sizeHint(self).width())
             self.tab.setMaximumHeight(1.3 * qt.QWidget.sizeHint(self).height())
@@ -725,7 +887,11 @@ class QStackWidget(StackBase.StackBase,
             self.rgbWidget = RGBCorrelator.RGBCorrelator()
             self.tab.addTab(self.rgbWidget, "RGB Correlator")
             self.mainLayout.addWidget(self.tab)
-        self.mcaWidget.setMiddleROIMarkerFlag(True)
+
+        curvesRoiWidget = self.mcaWidget.getCurvesRoiDockWidget().roiWidget
+        if hasattr(curvesRoiWidget, "setMiddleROIMarkerFlag"):
+            # Not implemented prior to silx >= 0.7
+            curvesRoiWidget.setMiddleROIMarkerFlag(True)
 
     def _buildAndConnectButtonBox(self):
         #the MCA selection
@@ -767,10 +933,11 @@ class QStackWidget(StackBase.StackBase,
         # self.stackGraphWidget.setInfoText("    X = ???? Y = ???? Z = ????")
         # self.stackGraphWidget.showInfo()
 
-        self.stackGraphWidget.graph.sigPlotSignal.connect(\
+        self.stackGraphWidget.graph.sigPlotSignal.connect( \
                                     self._stackGraphSignal)
+
         self.mcaWidget.sigROISignal.connect(self._mcaWidgetSignal)
-        self.roiWidget.graphWidget.graph.sigPlotSignal.connect(\
+        self.roiWidget.graphWidget.graph.sigPlotSignal.connect( \
                                     self._stackGraphSignal)
 
     def showOriginalImage(self):
@@ -778,7 +945,11 @@ class QStackWidget(StackBase.StackBase,
         if self._stackImageData is None:
             self.stackGraphWidget.graph.clear()
             return
-        self.stackWidget.setImageData(self._stackImageData)
+        xScale = self._stack.info.get("xScale", None)
+        yScale = self._stack.info.get("yScale", None)
+        self.stackWidget.setImageData(self._stackImageData,
+                                      xScale=xScale,
+                                      yScale=yScale)
 
     def showOriginalMca(self):
         goodData = numpy.isfinite(self._mcaData0.y[0].sum())
@@ -790,7 +961,7 @@ class QStackWidget(StackBase.StackBase,
         msg = qt.QMessageBox(self)
         msg.setIcon(qt.QMessageBox.Information)
         msg.setWindowTitle("Non finite data")
-        text  = "Your data contain infinite values or nans.\n"
+        text = "Your data contain infinite values or nans.\n"
         text += "Pixels containing those values will be ignored."
         msg.setText(text)
         msg.exec_()
@@ -804,7 +975,7 @@ class QStackWidget(StackBase.StackBase,
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Information)
             msg.setWindowTitle("No active curve selected")
-            text  = "Please select the MCA active curve."
+            text = "Please select the MCA active curve."
             msg.setText(text)
             msg.exec_()
             return
@@ -816,13 +987,19 @@ class QStackWidget(StackBase.StackBase,
                                                       energy=x)
 
     def showROIImageList(self, imageList, image_names=None):
+        xScale = self._stack.info.get("xScale", None)
+        yScale = self._stack.info.get("yScale", None)
         if self.roiBackgroundButton.isChecked():
-            self.roiWidget.graphWidget.graph.setGraphTitle(image_names[0]+\
-                                                      " Net")
-            self.roiWidget.setImageData(imageList[0]-imageList[-1])
+            self.roiWidget.graphWidget.graph.setGraphTitle(image_names[0] + \
+                                                           " Net")
+            self.roiWidget.setImageData(imageList[0]-imageList[-1],
+                                        xScale=xScale,
+                                        yScale=yScale)
         else:
             self.roiWidget.graphWidget.graph.setGraphTitle(image_names[0])
-            self.roiWidget.setImageData(imageList[0])
+            self.roiWidget.setImageData(imageList[0],
+                                        xScale=xScale,
+                                        yScale=yScale)
         self._ROIImageList = imageList
         self._ROIImageNames = image_names
         self._stackROIImageListUpdated()
@@ -885,18 +1062,18 @@ class QStackWidget(StackBase.StackBase,
 
         if self.normalizeButton.isChecked():
             if self._selectionMask is None:
-                npixels = self._stackImageData.shape[0] *\
+                npixels = self._stackImageData.shape[0] * \
                           self._stackImageData.shape[1]
             else:
                 npixels = self._selectionMask.sum()
                 if npixels == 0:
-                    npixels = self._stackImageData.shape[0] *\
+                    npixels = self._stackImageData.shape[0] * \
                               self._stackImageData.shape[1]
             legend += "/%d" % npixels
         return self.sendMcaSelection(dataObject,
-                          key = "Selection",
-                          legend =legend,
-                          action = action)
+                                     key="Selection",
+                                     legend=legend,
+                                     action=action)
 
     def _removeMcaClicked(self):
         #remove the mca
@@ -911,7 +1088,7 @@ class QStackWidget(StackBase.StackBase,
                 if curve.startswith(legend):
                     legend = curve
                     break
-        self.sendMcaSelection(dataObject, legend = legend, action = "REMOVE")
+        self.sendMcaSelection(dataObject, legend=legend, action="REMOVE")
 
     def _replaceMcaClicked(self):
         #replace the mca
@@ -919,7 +1096,7 @@ class QStackWidget(StackBase.StackBase,
         self._addMcaClicked(action="REPLACE")
         self.__ROIConnected = True
 
-    def sendMcaSelection(self, mcaObject, key = None, legend = None, action = None):
+    def sendMcaSelection(self, mcaObject, key=None, legend=None, action=None):
         if action is None:
             action = "ADD"
         if key is None:
@@ -932,9 +1109,9 @@ class QStackWidget(StackBase.StackBase,
                 legend += "/%d" % npixels
         sel = {}
         sel['SourceName'] = "EDF Stack"
-        sel['Key']        =  key
-        sel['legend']     =  legend
-        sel['dataobject'] =  mcaObject
+        sel['Key'] = key
+        sel['legend'] = legend
+        sel['dataobject'] = mcaObject
         if action == "ADD":
             self.mcaWidget._addSelection([sel])
         elif action == "REMOVE":
@@ -954,11 +1131,10 @@ class QStackWidget(StackBase.StackBase,
         if instance_id == id(self):
             return
 
-        if DEBUG:
-            if self._slave is not None:
-                print("MASTER  setSelectionMask CALLED")
-            elif self._masterStack is not None:
-                print("SLAVE setSelectionMask CALLED")
+        if self._slaveList is not None:
+            _logger.debug("MASTER  setSelectionMask CALLED")
+        elif self._masterStack is not None:
+            _logger.debug("SLAVE setSelectionMask CALLED")
 
         #inform built in widgets
         for widget in [self.stackWidget, self.roiWidget]:
@@ -969,18 +1145,18 @@ class QStackWidget(StackBase.StackBase,
                     widget.setSelectionMask(mask, plot=True)
 
         #inform slave
-        if self._slave is not None:
+        if self._slaveList is not None:
             #This is a master instance
-            instanceList = [id(self._slave),
-                            id(self._slave.stackWidget),
-                            id(self._slave.roiWidget)]
-            for key in self._slave.pluginInstanceDict.keys():
-                instanceList.append(id(self._slave.pluginInstanceDict[key]))
-            if instance_id not in instanceList:
-                #Originated by the master
-                if DEBUG:
-                    print("INFORMING SLAVE")
-                self._slave.setSelectionMask(mask, instance_id=id(self))
+            for slave in self._slaveList:
+                instanceList = [id(slave),
+                                id(slave.stackWidget),
+                                id(slave.roiWidget)]
+                for key in slave.pluginInstanceDict.keys():
+                    instanceList.append(id(slave.pluginInstanceDict[key]))
+                if instance_id not in instanceList:
+                    #Originated by the master
+                    _logger.warning("INFORMING SLAVE")
+                    slave.setSelectionMask(mask, instance_id=id(self))
 
         if self._masterStack is not None:
             #This is a slave instance
@@ -990,8 +1166,7 @@ class QStackWidget(StackBase.StackBase,
                 instanceList.append(id(self.pluginInstanceDict[key]))
             if instance_id in instanceList:
                 #Originated by the slave
-                if DEBUG:
-                    print("INFORMING MASTER")
+                _logger.debug("INFORMING MASTER")
                 self._masterStack.setSelectionMask(mask, instance_id=id(self))
 
         #Inform plugins
@@ -1024,11 +1199,9 @@ class QStackWidget(StackBase.StackBase,
             return
         if ddict['event'] == "hFlipSignal":
             if ddict['id'] != id(self.stackWidget):
-                self.stackWidget.graph.invertYAxis(ddict['current'])
-                self.stackWidget.graph.replot()
+                self.stackWidget.graph.getYAxis().setInverted(ddict['current'])
             if ddict['id'] != id(self.roiWidget):
-                self.roiWidget.graph.invertYAxis(ddict['current'])
-                self.roiWidget.graph.replot()
+                self.roiWidget.graph.getYAxis().setInverted(ddict['current'])
             return
 
     def _stackGraphSignal(self, ddict):
@@ -1045,8 +1218,8 @@ class QStackWidget(StackBase.StackBase,
             x = min(int(x), limits[0]-1)
             y = min(int(y), limits[1]-1)
             z = self._stackImageData[x, y]
-            self.stackGraphWidget.setInfoText("    X = %d Y = %d Z = %.4g" %\
-                                               (y, x, z))
+            self.stackGraphWidget.setInfoText( \
+                    "    X = %d Y = %d Z = %.4g" % (y, x, z))
 
     def _mcaWidgetSignal(self, ddict):
         if not self.__ROIConnected:
@@ -1077,6 +1250,15 @@ class QStackWidget(StackBase.StackBase,
     def getGraphYLabel(self):
         return self.mcaWidget.getGraphYLabel()
 
+    def closeEvent(self, event):
+        if self._slaveList is not None:
+            self._closeSlave()
+        # Inform plugins
+        for key in self.pluginInstanceDict.keys():
+            self.pluginInstanceDict[key].stackClosed()
+        CloseEventNotifyingWidget.CloseEventNotifyingWidget.closeEvent(self, event)
+
+
 def test():
     #create a dummy stack
     nrows = 100
@@ -1086,27 +1268,22 @@ def test():
     stackData = numpy.zeros((nrows, ncols, nchannels), numpy.float)
     for i in range(nchannels):
         stackData[:, :, i] = a * i
-    stackData[0:10,:,:] = 0
+    stackData[0:10, :, :] = 0
     w = QStackWidget()
     w.setStack(stackData, mcaindex=2)
     w.show()
     return w
 
+
 if __name__ == "__main__":
     sys.excepthook = qt.exceptionHandler
-    import getopt
-    options = ''
-    longoptions = ["fileindex=","old",
-                   "filepattern=", "begin=", "end=", "increment=",
-                   "nativefiledialogs=", "imagestack=", "image=",
-                   "backend="]
     try:
         opts, args = getopt.getopt(
                      sys.argv[1:],
                      options,
                      longoptions)
     except:
-        print(sys.exc_info()[1])
+        print("%s" % sys.exc_info()[1])
         sys.exit(1)
     fileindex = 0
     filepattern=None
@@ -1116,6 +1293,7 @@ if __name__ == "__main__":
     increment=None
     backend=None
     PyMcaDirs.nativeFileDialogs=True
+
     for opt, arg in opts:
         if opt in '--begin':
             if "," in arg:
@@ -1133,17 +1311,17 @@ if __name__ == "__main__":
             else:
                 increment = int(arg)
         elif opt in '--filepattern':
-            filepattern = arg.replace('"','')
-            filepattern = filepattern.replace("'","")
+            filepattern = arg.replace('"', '')
+            filepattern = filepattern.replace("'", "")
         elif opt in '--fileindex':
             fileindex = int(arg)
         elif opt in ['--imagestack', "--image"]:
             imagestack = int(arg)
         elif opt in '--nativefiledialogs':
             if int(arg):
-                PyMcaDirs.nativeFileDialogs=True
+                PyMcaDirs.nativeFileDialogs = True
             else:
-                PyMcaDirs.nativeFileDialogs=False
+                PyMcaDirs.nativeFileDialogs = False
         elif opt in '--backend':
             backend = arg
         #elif opt in '--old':
@@ -1159,10 +1337,10 @@ if __name__ == "__main__":
     if backend is not None:
         # set the default backend
         try:
-            from PyMca5.PyMcaGraph.Plot import Plot
-            Plot.defaultBackend = backend
+            from silx.gui.plot import PlotWidget
+            PlotWidget.setDefaultBackend(backend)
         except:
-            print("WARNING: Cannot set backend to %s" % backend)
+            _logger.warning("WARNING: Cannot set backend to %s", backend)
     widget = QStackWidget()
     w = StackSelector.StackSelector(widget)
     if filepattern is not None:
@@ -1171,12 +1349,16 @@ if __name__ == "__main__":
                                       imagestack=imagestack)
     else:
         stack = w.getStack(args, imagestack=imagestack)
-    if type(stack) == type([]):
+    if (type(stack) == type([])) or (isinstance(stack, list)):
         #aifira like, two stacks
         widget.setStack(stack[0])
-        slave = QStackWidget(master=False, rgbwidget=widget.rgbWidget)
-        slave.setStack(stack[1])
-        widget.setSlave(slave)
+        if len(stack) > 1:
+            for i in range(1, len(stack)):
+                if stack[i] is not None:
+                    slave = QStackWidget(master=False,
+                                         rgbwidget=widget.rgbWidget)
+                    slave.setStack(stack[i])
+                    widget.addSlave(slave)
         stack = None
     else:
         widget.setStack(stack)
