@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2015 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2018 V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -31,6 +31,8 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import os
 import copy
+import traceback
+import logging
 from PyMca5.PyMcaGui import PyMcaQt as qt
 from PyMca5 import PyMcaDirs
 from PyMca5 import DataObject
@@ -46,6 +48,8 @@ from PyMca5.PyMcaIO import TiffStack
 from PyMca5.PyMcaIO import RTXMap
 from PyMca5.PyMcaIO import LispixMap
 from PyMca5.PyMcaIO import RenishawMap
+from PyMca5.PyMcaIO import OmdaqLmf
+from PyMca5.PyMcaIO import JcampOpusStack
 from .QStack import QStack, QSpecFileStack
 try:
     from PyMca5.PyMcaGui.pymca import QHDF5Stack1D
@@ -56,7 +60,7 @@ except ImportError:
     pass
 
 QTVERSION = qt.qVersion()
-DEBUG = 0
+_logger = logging.getLogger(__name__)
 
 
 class StackSelector(object):
@@ -183,6 +187,12 @@ class StackSelector(object):
                 # that case as a stack.
                 stack = RenishawMap.RenishawMap(filelist[0])
                 omnicfile = True
+            elif OmdaqLmf.isOmdaqLmf(filelist[0]):
+                stack = OmdaqLmf.OmdaqLmf(filelist[0])
+                omnicfile = True
+            elif JcampOpusStack.isJcampOpusStackFile(filelist[0]):
+                stack = JcampOpusStack.JcampOpusStack(filelist[0])
+                omnicfile = True
             else:
                 stack = QSpecFileStack()
 
@@ -194,12 +204,10 @@ class StackSelector(object):
                 except:
                     msg = qt.QMessageBox()
                     msg.setIcon(qt.QMessageBox.Critical)
-                    msg.setText("%s" % sys.exc_info()[1])
-                    if QTVERSION < '4.0.0':
-                        msg.exec_loop()
-                    else:
-                        msg.exec_()
-                    if DEBUG:
+                    msg.setInformativeText("%s" % sys.exc_info()[1])
+                    msg.setDetailedText(traceback.format_exc())
+                    msg.exec_()
+                    if _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
         elif len(filelist):
             if not omnicfile:
@@ -213,7 +221,7 @@ class StackSelector(object):
                         msg.exec_loop()
                     else:
                         msg.exec_()
-                    if DEBUG:
+                    if _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
         if aifirafile:
             masterStack = DataObject.DataObject()
@@ -244,6 +252,9 @@ class StackSelector(object):
         #read 10 characters
         line = f.read(10)
         f.close()
+        if hasattr(line, "decode"):
+            # convert to string ignoring errors
+            line = line.decode("utf-8", "ignore")
 
         specfile = False
         marCCD = False
@@ -251,7 +262,7 @@ class StackSelector(object):
             marCCD = True
         if line[0] == "\n":
             line = line[1:]
-        if (line[0] == "{") or marCCD:
+        if line.startswith("{") or marCCD:
             if imagestack is None:
                 if marCCD:
                     imagestack = True
@@ -272,7 +283,7 @@ class StackSelector(object):
             stack = SupaVisioMap.SupaVisioMap(args[0])
         elif args[0][-3:].upper() in ["RBS"]:
             stack = SupaVisioMap.SupaVisioMap(args[0])
-        elif args[0][-3:].lower() in [".h5", "nxs", "hdf"]:
+        elif args[0][-3:].lower() in [".h5", "nxs", "hdf", "hdf5"]:
             if not HDF5:
                 raise IOError(\
                     "No HDF5 support while trying to read an HDF5 file")
@@ -346,17 +357,17 @@ class StackSelector(object):
     def _getStackOfFiles(self, getfilter=None):
         if getfilter is None:
             getfilter = False
-        fileTypeList = ["EDF Files (*edf)",
+        fileTypeList = ["EDF Files (*edf *edf.gz)",
                         "Image Files (*edf *ccd *raw *edf.gz *ccd.gz *raw.gz *cbf)",
                         "Image Files (*tif *tiff *TIF *TIFF)",
                         "TextImage Files (*txt)",
-                        "HDF5 Files (*.nxs *.hdf *.h5)",
+                        "HDF5 Files (*.nxs *.hdf *.hdf5 *.h5)",
                         "EDF Files (*ccd)",
                         "Specfile Files (*mca)",
                         "Specfile Files (*dat)",
                         "OMNIC Files (*map)",
                         "OPUS-DPT Files (*.DPT *.dpt)",
-                        "JCAMP-DX Files (*.JDX *.jdx)",
+                        "JCAMP-DX Files (*.JDX *.jdx *.DX *.dx)",
                         "RTX Files (*.rtx *.RTX)",
                         "Lispix-RPL Files (*.rpl)",
                         "Renishaw-ASCII Files (*.txt *.TXT)",
@@ -365,7 +376,7 @@ class StackSelector(object):
                         "MRC files (*.mrc *.st)",
                         "All Files (*)"]
         if not HDF5:
-            idx = fileTypeList.index("HDF5 Files (*.nxs *.hdf *.h5)")
+            idx = fileTypeList.index("HDF5 Files (*.nxs *.hdf *.hdf5 *.h5)")
             del fileTypeList[idx]
         message = "Open ONE indexed stack or SEVERAL files"
         return self._getFileList(fileTypeList, message=message,
@@ -418,7 +429,7 @@ if __name__ == "__main__":
                      options,
                      longoptions)
     except:
-        print(sys.exc_info()[1])
+        _logger.error(sys.exc_info()[1])
         sys.exit(1)
     fileindex = 0
     filepattern = None
