@@ -37,6 +37,7 @@ from PyMca5.PyMcaMisc import PhysicalMemory
 import numpy
 import sys
 import os
+import logging
 
 # Offer automatic conversion to HDF5 in case of lacking
 # memory to hold the Stack.
@@ -49,7 +50,8 @@ except:
 
 
 SOURCE_TYPE = "EdfFileStack"
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 X_AXIS=0
 Y_AXIS=1
@@ -232,9 +234,9 @@ class EDFStack(DataObject.DataObject):
                             samplingStep = None
                             i = 2
                             while samplingStep is None:
-                                print("**************************************************")
-                                print(" Memory error!, attempting %dx%d sampling reduction ") % (i,i)
-                                print("**************************************************")
+                                _logger.warning("**************************************************")
+                                _logger.warning(" Memory error!, attempting %dx%d sampling reduction ", i,i)
+                                _logger.warning("**************************************************")
                                 s1, s2 = arrRet[::i, ::i].shape
                                 try:
                                     self.data = numpy.zeros((s1, s2,
@@ -431,6 +433,14 @@ class EDFStack(DataObject.DataObject):
                                 if os.path.exists(i0EndFile):
                                     i0End = EdfFile.EdfFile(i0EndFile, 'rb').GetData(0) - bckData
                                     i0Slope = (i0End-i0Start)/len(filelist)
+                            positionersFile = filelist[0].replace("_sample_", "_positioners_")
+                            if os.path.exists(positionersFile):
+                                positionersEdf = EdfFile.EdfFile(positionersFile, 'rb')
+                                self.info["positioners"] = {}
+                                for i in range(positionersEdf.GetNumImages()):
+                                    motorName = positionersEdf.GetHeader(i).get("Title", "Motor_%02d" % i)
+                                    motorValue = positionersEdf.GetData(i)
+                                    self.info["positioners"][motorName] = motorValue
                         for tempEdfFileName in filelist:
                             tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                             if ID24:
@@ -443,14 +453,14 @@ class EDFStack(DataObject.DataObject):
                                 self.data[self.incrProgressBar, :,:] = pieceOfStack[:,:]
                             except:
                                 if pieceOfStack.shape[1] != arrRet.shape[1]:
-                                    print(" ERROR on file %s" % tempEdfFileName)
-                                    print(" DIM 1 error Assuming missing data were at the end!!!")
+                                    _logger.warning(" ERROR on file %s", tempEdfFileName)
+                                    _logger.warning(" DIM 1 error Assuming missing data were at the end!!!")
                                 if pieceOfStack.shape[0] != arrRet.shape[0]:
-                                    print(" ERROR on file %s" % tempEdfFileName)
-                                    print(" DIM 0 error Assuming missing data were at the end!!!")
-                                self.data[self.incrProgressBar,\
-                                         :pieceOfStack.shape[0],\
-                                         :pieceOfStack.shape[1]] = pieceOfStack[:,:]
+                                    _logger.warning(" ERROR on file %s", tempEdfFileName)
+                                    _logger.warning(" DIM 0 error Assuming missing data were at the end!!!")
+                                self.data[self.incrProgressBar,
+                                          :pieceOfStack.shape[0],
+                                          :pieceOfStack.shape[1]] = pieceOfStack[:, :]
                             self.incrProgressBar += 1
                             self.onProgress(self.incrProgressBar)
                     self.onEnd()
@@ -486,6 +496,28 @@ class EDFStack(DataObject.DataObject):
             self.info["NumberOfFiles"] = self.__nFiles * 1
             self.info["Size"] = self.__nFiles * self.__nImagesPerFile
 
+        # try to use positioners to compute the scales (ID24 specific)
+        xPositionerName = None
+        yPositionerName = None
+        if "positioners" in self.info and len(self.info["positioners"]) == 2:
+            for k, v in self.info["positioners"].items():
+                if isinstance(v, numpy.ndarray) and v.ndim == 2:
+                    deltaDim1 = v[:, 1:] - v[:, :-1]
+                    deltaDim0 = v[1:, :] - v[:-1, :]
+                    if numpy.any(deltaDim1) and not numpy.any(deltaDim0):
+                        # positioner varying only along dim1
+                        xPositionerName = k
+                        # should we check that all delta values are equal?
+                        deltaX = numpy.mean(deltaDim1)
+                        originX = v[0, 0]
+                    elif numpy.any(deltaDim0) and not numpy.any(deltaDim1):
+                        # positioner varying only along dim0
+                        yPositionerName = k
+                        deltaY = numpy.mean(deltaDim0)
+                        originY = v[0, 0]
+            if xPositionerName is not None and yPositionerName is not None:
+                self.info["xScale"] = (originX, deltaX)
+                self.info["yScale"] = (originY, deltaY)
 
     def onBegin(self, n):
         pass
@@ -541,8 +573,9 @@ class EDFStack(DataObject.DataObject):
                 prefix = name[0:n-i+1]
             prefix = os.path.join(os.path.dirname(filename),prefix)
             if not os.path.exists(prefix + number + suffix):
-                print("Internal error in EDFStack")
-                print("file should exist: %s " % (prefix + number + suffix))
+                _logger.error("Internal error in EDFStack "
+                              "file should exist: %s ",
+                              prefix + number + suffix)
                 return
             i = 0
             if begin is None:
@@ -579,7 +612,7 @@ class EDFStack(DataObject.DataObject):
         sourceInfo["KeyList"]= self.__keyList
 
     def getKeyInfo(self, key):
-        print("Not implemented")
+        _logger.info("Not implemented")
         return {}
 
     def isIndexedStack(self):
