@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2016 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -35,6 +35,7 @@ Module to perform a fast linear fit on a stack of fluorescence spectra.
 """
 import os
 import numpy
+import logging
 from PyMca5.PyMcaMath.linalg import lstsq
 from . import ClassMcaTheory
 from PyMca5.PyMcaMath.fitting import Gefit
@@ -43,7 +44,8 @@ from PyMca5.PyMcaMath.fitting import SpecfitFuns
 from PyMca5.PyMcaIO import ConfigDict
 import time
 
-DEBUG = 0
+_logger = logging.getLogger(__name__)
+
 
 class FastXRFLinearFit(object):
     def __init__(self, mcafit=None):
@@ -55,6 +57,7 @@ class FastXRFLinearFit(object):
 
     def setFitConfiguration(self, configuration):
         self._mcaTheory.setConfiguration(configuration)
+        self._config = self._mcaTheory.getConfiguration()
 
     def setFitConfigurationFile(self, ffile):
         if not os.path.exists(ffile):
@@ -65,7 +68,8 @@ class FastXRFLinearFit(object):
 
     def fitMultipleSpectra(self, x=None, y=None, xmin=None, xmax=None,
                            configuration=None, concentrations=False,
-                           ysum=None, weight=None, refit=True):
+                           ysum=None, weight=None, refit=True,
+                           livetime=None):
         """
         This method performs the actual fit. The y keyword is the only mandatory input argument.
 
@@ -76,32 +80,94 @@ class FastXRFLinearFit(object):
         :param weight: 0 Means no weight, 1 Use an average weight, 2 Individual weights (slow)
         :param concentrations: 0 Means no calculation, 1 Calculate them
         :param refit: if False, no check for negative results. Default is True.
+        :livetime: It will be used if not different from None and concentrations
+                   are to be calculated by using fundamental parameters with
+                   automatic time. The default is None.
         :return: A dictionnary with the parameters, uncertainties, concentrations and names as keys.
         """
         if y is None:
             raise RuntimeError("y keyword argument is mandatory!")
 
-        #if concentrations:
-        #    txt = "Fast concentration calculation not implemented yet"
-        #    raise NotImplemented(txt)
+        if hasattr(y, "info") and hasattr(y, "data"):
+            data = y.data
+            mcaIndex = y.info.get("McaIndex", -1)
+        else:
+            data = y
+            mcaIndex = -1
 
-        if DEBUG:
-            t0 = time.time()
+        if x is None:
+            if hasattr(y, "info") and hasattr(y, "x"):
+                x = y.x[0]
+
+        if livetime is None:
+            if hasattr(y, "info"):
+                if "McaLiveTime" in y.info:
+                    livetime = y.info["McaLiveTime"]
+        t0 = time.time()
         if configuration is not None:
             self._mcaTheory.setConfiguration(configuration)
-
+        elif self._config is None:
+            raise ValueError("Fit configuration missing")
+        else:
+            _logger.debug("Setting default configuration")
+            self._mcaTheory.setConfiguration(self._config)
         # read the current configuration
+        # it is a copy, we can modify it at will
         config = self._mcaTheory.getConfiguration()
+        if xmin is None:
+            xmin = config['fit']['xmin']
+        if xmax is None:
+            xmax = config['fit']['xmax']
+        toReconfigure = False
+
+        # if concentrations and use times, it needs to be reconfigured
+        # without using times and correct later on. If the concentrations
+        # are to be calculated from internal standard there is no need to
+        # raise an exception either.
+        autotime = 0
+        liveTimeFactor = 1.0
+        if not concentrations:
+            # ignore any time information to prevent unnecessary errors when
+            # setting the fitting data whithout the time information
+            if config['concentrations'].get("useautotime", 0):
+                config['concentrations']["useautotime"] = 0
+                toReconfigure = True
+        elif config["concentrations"]["usematrix"]:
+            if config['concentrations'].get("useautotime", 0):
+                config['concentrations']["useautotime"] = 0
+                toReconfigure = True
+        else:
+            # we are calculating concentrations from fundamental parameters
+            autotime = config['concentrations'].get("useautotime", 0)
+            nSpectra = data.size // data.shape[mcaIndex]
+            if autotime:
+                if livetime is None:
+                    txt = "Automatic time requested but no time information provided"
+                    raise RuntimeError(txt)
+                elif numpy.isscalar(livetime):
+                    liveTimeFactor = \
+                        float(config['concentrations']["time"]) / livetime
+                elif livetime.size == nSpectra:
+                    liveTimeFactor = \
+                        float(config['concentrations']["time"]) / livetime
+                else:
+                    raise RuntimeError( \
+                        "Number of live times not equal number of spectra")
+                config['concentrations']["useautotime"] = 0
+                toReconfigure = True
+
+        # use of strategies is not supported for the time being
+        strategy = config['fit'].get('strategyflag', 0)
+        if strategy:
+            raise RuntimeError("Strategies are incompatible with fast fit")
 
         # background
         if config['fit']['stripflag']:
             if config['fit']['stripalgorithm'] == 1:
-                if DEBUG:
-                    print("SNIP")
+                _logger.debug("SNIP")
             else:
                 raise RuntimeError("Please use the faster SNIP background")
 
-        toReconfigure = False
         if weight is None:
             # dictated by the file
             weight = config['fit']['fitweight']
@@ -141,13 +207,6 @@ class FastXRFLinearFit(object):
             # we must configure again the fit
             self._mcaTheory.setConfiguration(config)
 
-        if hasattr(y, "info") and hasattr(y, "data"):
-            data = y.data
-            mcaIndex = y.info.get("McaIndex", -1)
-        else:
-            data = y
-            mcaIndex = -1
-
         if len(data.shape) != 3:
             txt = "For the time being only three dimensional arrays supported"
             raise IndexError(txt)
@@ -207,6 +266,10 @@ class FastXRFLinearFit(object):
                 freeNames.append(param)
                 if i < self._mcaTheory.NGLOBAL:
                     nFreeBackgroundParameters += 1
+        if nFree == 0:
+            txt = "No free parameters to be fitted!\n"
+            txt += "No peaks inside fitting region?"
+            raise ValueError(txt)
 
         #build the matrix of derivatives
         derivatives = None
@@ -255,7 +318,7 @@ class FastXRFLinearFit(object):
             iXMax = numpy.nonzero(x >= xdata[-1])[0][0]
         # numpy 1.11.0 returns an array on previous expression
         # and then complains about a future deprecation warning
-        # because of using an arry and not an scalar in the selection
+        # because of using an array and not an scalar in the selection
         if hasattr(iXMin, "shape"):
             if len(iXMin.shape):
                 iXMin = iXMin[0]
@@ -271,9 +334,8 @@ class FastXRFLinearFit(object):
         uncertainties = numpy.zeros((nFree, nRows, nColumns), numpy.float32)
 
         #perform the initial fit
-        if DEBUG:
-            print("Configuration elapsed = %f"  % (time.time() - t0))
-            t0 = time.time()
+        _logger.debug("Configuration elapsed = %f", time.time() - t0)
+        t0 = time.time()
         totalSpectra = data.shape[0] * data.shape[1]
         jStep = min(100, data.shape[1])
         if weightPolicy == 2:
@@ -333,11 +395,12 @@ class FastXRFLinearFit(object):
                 results[:, i, jStart:jEnd] = parameters
                 uncertainties[:, i, jStart:jEnd] = ddict['uncertainties']
                 jStart = jEnd
-        if DEBUG:
-            t = time.time() - t0
-            print("First fit elapsed = %f" % t)
-            print("Spectra per second = %f" % (data.shape[0]*data.shape[1]/float(t)))
-            t0 = time.time()
+        t = time.time() - t0
+        _logger.debug("First fit elapsed = %f", t)
+        if t > 0.:
+            _logger.debug("Spectra per second = %f",
+                          data.shape[0]*data.shape[1]/float(t))
+        t0 = time.time()
 
         # cleanup zeros
         # start with the parameter with the largest amount of negative values
@@ -370,7 +433,8 @@ class FastXRFLinearFit(object):
                     i = item[1]
                     badMask = item[2]
                     results[i][badMask] = 0.0
-                    print("WARNING: %d pixels of parameter %s forced to zero" % (item[0], freeNames[i]))
+                    _logger.warning("WARNING: %d pixels of parameter %s forced to zero",
+                                    item[0], freeNames[i])
                 continue
             zeroList.sort()
             zeroList.reverse()
@@ -394,17 +458,13 @@ class FastXRFLinearFit(object):
                 for i in badParameters:
                     results[i][badMask] = 0.0
                     uncertainties[i][badMask] = 0.0
-                    if DEBUG:
-                        print("WARNING: %d pixels of parameter %s set to zero" % (badMask.sum(),
-                                                                                  freeNames[i]))
+                    _logger.debug("WARNING: %d pixels of parameter %s set to zero",
+                                  badMask.sum(), freeNames[i])
             else:
-                if DEBUG:
-                    print("Number of secondary fits = %d" % (nFits + 1))
+                _logger.debug("Number of secondary fits = %d", nFits + 1)
                 nFits += 1
                 A = derivatives[:, [i for i in range(nFree) if i not in badParameters]]
                 #assume we'll not have too many spectra
-                if data.dtype not in [numpy.float32, numpy.float64]:
-                    data
                 if data.dtype not in [numpy.float32, numpy.float64]:
                     if data.itemsize < 5:
                         data_dtype = numpy.float32
@@ -455,7 +515,7 @@ class FastXRFLinearFit(object):
                                     SpecfitFuns.snip1d(background[lastAnchor:],
                                                        config['fit']['snipwidth'],
                                                        0)
-                    spectra[:, k] -= background
+                        spectra[:, k] -= background
                 ddict = lstsq(A, spectra,
                               sigma_b=sigma_b,
                               weight=weight,
@@ -471,9 +531,9 @@ class FastXRFLinearFit(object):
                         uncertainties[i][badMask] = ddict['uncertainties'][idx]
                         idx += 1
 
-        if DEBUG and refit:
+        if refit:
             t = time.time() - t0
-            print("Fit of negative peaks elapsed = %f" % t)
+            _logger.debug("Fit of negative peaks elapsed = %f", t)
             t0 = time.time()
 
         outputDict = {'parameters':results, 'uncertainties':uncertainties, 'names':freeNames}
@@ -488,10 +548,13 @@ class FastXRFLinearFit(object):
 
             fitFirstSpectrum = False
             if config['concentrations']['usematrix']:
-                if DEBUG:
-                    print("USING MATRIX")
+                _logger.debug("USING MATRIX")
                 if config['concentrations']['reference'].upper() == "AUTO":
                     fitFirstSpectrum = True
+            elif autotime:
+                # we have to calculate with the time in the configuration
+                # and correct later on
+                cToolConf["autotime"] = 0
 
             fitresult = {}
             if fitFirstSpectrum:
@@ -545,30 +608,34 @@ class FastXRFLinearFit(object):
 
             referenceElement = addInfo['ReferenceElement']
             referenceTransitions = addInfo['ReferenceTransitions']
-            if DEBUG:
-                print("Reference <%s>  transition <%s>" % (referenceElement, referenceTransitions))
+            _logger.debug("Reference <%s>  transition <%s>",
+                          referenceElement, referenceTransitions)
             if referenceElement in ["", None, "None"]:
-                if DEBUG:
-                    print("No reference")
+                _logger.debug("No reference")
                 counter = 0
                 for i, group in enumerate(fitresult['result']['groups']):
                     if group.lower().startswith("scatter"):
-                        if DEBUG:
-                            print("skept %s" % group)
+                        _logger.debug("skept %s", group)
                         continue
                     outputDict['names'].append("C(%s)" % group)
-                    massFractions[counter] = results[nFreeBackgroundParameters+i] *\
-                        (concentrationsResult['mass fraction'][group]/fitresult['result'][group]['fitarea'])
+                    if counter == 0:
+                        if hasattr(liveTimeFactor, "shape"):
+                            liveTimeFactor.shape = results[nFreeBackgroundParameters+i].shape
+                    massFractions[counter] = liveTimeFactor * \
+                        results[nFreeBackgroundParameters+i] * \
+                        (concentrationsResult['mass fraction'][group] / \
+                         fitresult['result'][group]['fitarea'])
                     counter += 1
                     if len(concentrationsResult['layerlist']) > 1:
                         for layer in concentrationsResult['layerlist']:
                             outputDict['names'].append("C(%s)-%s" % (group, layer))
-                            massFractions[counter] = results[nFreeBackgroundParameters+i] *\
-                        (concentrationsResult[layer]['mass fraction'][group]/fitresult['result'][group]['fitarea'])
+                            massFractions[counter] = liveTimeFactor * \
+                                    results[nFreeBackgroundParameters+i] * \
+                                    (concentrationsResult[layer]['mass fraction'][group] / \
+                                     fitresult['result'][group]['fitarea'])
                             counter += 1
             else:
-                if DEBUG:
-                    print("With reference")
+                _logger.debug("With reference")
                 idx = None
                 testGroup = referenceElement+ " " + referenceTransitions.split()[0]
                 for i, group in enumerate(fitresult['result']['groups']):
@@ -586,8 +653,7 @@ class FastXRFLinearFit(object):
                 counter = 0
                 for i, group in enumerate(fitresult['result']['groups']):
                     if group.lower().startswith("scatter"):
-                        if DEBUG:
-                            print("skept %s" % group)
+                        _logger.debug("skept %s", group)
                         continue
                     outputDict['names'].append("C(%s)" % group)
                     goodI = results[nFreeBackgroundParameters+i] > 0
@@ -604,10 +670,8 @@ class FastXRFLinearFit(object):
                                 (concentrationsResult[layer]['mass fraction'][group]))
                             counter += 1
             outputDict['concentrations'] = massFractions
-            if DEBUG:
-                t = time.time() - t0
-                print("Calculation of concentrations elapsed = %f" % t)
-                t0 = time.time()
+            t = time.time() - t0
+            _logger.debug("Calculation of concentrations elapsed = %f", t)
             ####################################################
         return outputDict
 
@@ -645,7 +709,8 @@ def getFileListFromPattern(pattern, begin, end, increment=None):
     return fileList
 
 if __name__ == "__main__":
-    DEBUG = True
+    logging.basicConfig(level=logging.INFO)
+    _logger.setLevel(logging.DEBUG)
     import glob
     import sys
     from PyMca5.PyMca import EDFStack
@@ -720,34 +785,38 @@ if __name__ == "__main__":
     if refit is None:
         refit = 0
         print("WARNING: --refit=%d taken as default" % refit)
-    if 0:
-        #configurationFile = "E:\DATA\COTTE\CH1777\G4-4720eV-NOWEIGHT-NO_Constant-batch.cfg"
-        #configurationFile = "E:\DATA\COTTE\CH1777\G4-4720eV-NOWEIGHT-NO_Constant-batch.cfg"
-        configurationFile = "E:\DATA\COTTE\CH1777\G4-4720eV-SINGLELAYER-batch.cfg"
-        configurationFile = "E:\DATA\COTTE\CH1777\G4-4720eV-SINGLELAYER-Auto-batch.cfg"
-        #configurationFile = "E:\DATA\COTTE\CH1777\G4-4720eV-MULTILAYER-batch.cfg"
-        configurationFile = "E:\DATA\COTTE\CH1777\G4-4720eV-MULTILAYER-Pb-batch.cfg"
-        fileList = glob.glob("E:\DATA\COTTE\CH1777\G4_mca_0012_0000_*.edf")
-        concentrations = True
-        dataStack = EDFStack.EDFStack(filelist=fileList)
-        outputDir = "E:\\CHTEST\\"
-    elif 0:
-        configurationFile = "E:\DATA\COTTE\CH09\WithLeadBatch.cfg"
-        fileList = glob.glob("E:\DATA\COTTE\CH09\ch09__mca_0005_0000_*.edf")
-        concentrations = False
-        dataStack = EDFStack.EDFStack(filelist=fileList)
-    elif 0:
-        configurationFile = "D:\RIVARD\config_3-6kev_OceanIsland_batch_NO_BACKGROUND.cfg"
-        fileList = glob.glob("D:\RIVARD\m*.edf")
-        concentrations = False
-        dataStack = EDFStack.EDFStack(filelist=fileList)
-    elif 0:
-        configurationFile = "E2_line.cfg"
-        fileList = glob.glob("E:\DATA\PyMca-Training\FDM55\AS_EDF\E2_line*.edf")
-        concentrations = False
-        dataStack = EDFStack.EDFStack(filelist=fileList)
-    elif len(fileList):
-        dataStack = EDFStack.EDFStack(fileList, dtype=numpy.float32)
+    if len(fileList):
+        if (not os.path.exists(fileList[0])) and \
+           os.path.exists(fileList[0].split("::")[0]):
+            # odo convention to get a dataset form an HDF5
+            fname, dataPath = fileList[0].split("::")
+            # compared to the ROI imaging tool, this way of reading puts data
+            # into memory while with the ROI imaging tool, there is a check.
+            if 0:
+                import h5py
+                h5 = h5py.File(fname, "r")
+                dataStack = h5[dataPath][:]
+                h5.close()
+            else:
+                from PyMca5.PyMcaIO import HDF5Stack1D
+                # this way reads information associated to the dataset (if present)
+                if dataPath.startswith("/"):
+                    pathItems = dataPath[1:].split("/")
+                else:
+                    pathItems = dataPath.split("/")
+                if len(pathItems) > 1:
+                    scanlist = ["/" + pathItems[0]]
+                    selection = {"y":"/" + "/".join(pathItems[1:])}
+                else:
+                    selection = {"y":dataPath}
+                    scanlist = None
+                print(selection)
+                print("scanlist = ", scanlist)
+                dataStack = HDF5Stack1D.HDF5Stack1D([fname],
+                                                    selection,
+                                                    scanlist=scanlist)
+        else:
+            dataStack = EDFStack.EDFStack(fileList, dtype=numpy.float32)
     else:
         print("OPTIONS:", longoptions)
         sys.exit(0)
