@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2016 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -59,11 +59,15 @@ class McaAdvancedFitBatch(object):
                     concentrations=0, fitfiles=1, fitimages=1,
                     filebeginoffset = 0, fileendoffset=0,
                     mcaoffset=0, chunk = None,
-                    selection=None, lock=None):
+                    selection=None, lock=None, nosave=None, quiet=False):
         #for the time being the concentrations are bound to the .fit files
         #that is not necessary, but it will be correctly implemented in
         #future releases
         self._lock = lock
+        if nosave:
+            self._nosave = True
+        else:
+            self._nosave = False
         self.fitFiles = fitfiles
         self._concentrations = concentrations
         if type(initdict) == type([]):
@@ -100,6 +104,7 @@ class McaAdvancedFitBatch(object):
         self.mcaOffset = mcaoffset
         self.chunk     = chunk
         self.selection = selection
+        self.quiet = quiet
 
 
     def setFileList(self,filelist=None):
@@ -190,12 +195,13 @@ class McaAdvancedFitBatch(object):
                     break
             else:
                 self.__processOneFile()
+
         if self.counter:
             if not self.roiFit:
                 if self.fitFiles:
                     self.listfile.write(']\n')
                     self.listfile.close()
-            if self.__ncols is not None:
+            if (self.__ncols is not None) and (not self._nosave):
                 if self.__ncols:self.saveImage()
         self.onEnd()
 
@@ -215,8 +221,11 @@ class McaAdvancedFitBatch(object):
                 if h5py.is_hdf5(inputfile):
                     self._HDF5 = True
                     try:
+                        # if (len(self._filelist) == 1) && (self.mcaStep > 1)
+                        # it should attempt to avoid loading  many times
+                        # the stack into memory in case of multiple processes
                         return HDF5Stack1D.HDF5Stack1D(self._filelist,
-                                                      self.selection)
+                                                       self.selection)
                     except:
                         raise
             
@@ -239,7 +248,8 @@ class McaAdvancedFitBatch(object):
 
 
     def onNewFile(self,ffile, filelist):
-        self.__log(ffile)
+        if not self.quiet:
+            self.__log(ffile)
 
     def onImage(self,image,imagelist):
         pass
@@ -355,7 +365,10 @@ class McaAdvancedFitBatch(object):
                 infoDict = {}
                 infoDict['SourceName'] = info['SourceName']
                 infoDict['Key']        = key
-                self.__processOneMca(x,y0,filename,key,info=infoDict)
+                if "McaLiveTime" in info:
+                    infoDict["McaLiveTime"] = \
+                            info["McaLiveTime"][i * numberofmca + mca]
+                self.__processOneMca(x, y0, filename, key, info=infoDict)
                 self.onMca(mca, numberofmca, filename=filename,
                                             key=key,
                                             info=infoDict)
@@ -698,18 +711,20 @@ class McaAdvancedFitBatch(object):
                 #this only works with EDF
                 if self.__ncols is not None:
                     if not self.counter:
-                        imgdir = self.os_path_join(self._outputdir,"IMAGES")
-                        if not os.path.exists(imgdir):
-                            try:
-                                os.mkdir(imgdir)
-                            except:
-                                print("I could not create directory %s" %\
+                        if not self._nosave:
+                            imgdir = self.os_path_join(self._outputdir,"IMAGES")
+                            if not os.path.exists(imgdir):
+                                try:
+                                    os.mkdir(imgdir)
+                                except:
+                                    print("I could not create directory %s" %\
+                                          imgdir)
+                                    return
+                            elif not os.path.isdir(imgdir):
+                                print("%s does not seem to be a valid directory" %\
                                       imgdir)
-                                return
-                        elif not os.path.isdir(imgdir):
-                            print("%s does not seem to be a valid directory" %\
-                                  imgdir)
-                        self.imgDir = imgdir
+                            self.imgDir = imgdir
+
                         self.__peaks  = []
                         self.__images = {}
                         self.__sigmas = {}
@@ -774,19 +789,21 @@ class McaAdvancedFitBatch(object):
                 dict=self.mcafit.roifit(x,y,width=self.roiWidth)
                 #this only works with EDF
                 if self.__ncols is not None:
+                    self.imgDir=None
                     if not self.counter:
-                        imgdir = self.os_path_join(self._outputdir,"IMAGES")
-                        if not os.path.exists(imgdir):
-                            try:
-                                os.mkdir(imgdir)
-                            except:
-                                print("I could not create directory %s" %\
+                        if not self._nosave:
+                            imgdir = self.os_path_join(self._outputdir,"IMAGES")
+                            if not os.path.exists(imgdir):
+                                try:
+                                    os.mkdir(imgdir)
+                                except:
+                                    print("I could not create directory %s" %\
+                                          imgdir)
+                                    return
+                            elif not os.path.isdir(imgdir):
+                                print("%s does not seem to be a valid directory" %\
                                       imgdir)
-                                return
-                        elif not os.path.isdir(imgdir):
-                            print("%s does not seem to be a valid directory" %\
-                                  imgdir)
-                        self.imgDir = imgdir
+                            self.imgDir = imgdir
                         self.__ROIpeaks  = []
                         self._ROIimages = {}
                         if not self.__stack:
