@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2015 T. Rueter, V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2018 T. Rueter, V.A. Sole, European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -27,7 +27,111 @@ __author__ = "Tonn Rueter & V.A. Sole - ESRF Data Analysis"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
+__doc__ = """
+Due to uncertainties in the experimental set-up, recorded data might be shifted unrelated
+to physical effects probed in the experiment. The present plug-in calculates this shift and
+corrects the data using a variety of different methods.
+
+Usage and Description
+---------------------
+
+Data that is subject to a shift must be loaded into the plot window of the main application.
+The plug-in offers two ways to treat the data:
+
+ - A shortcut options, called *Perform FFT Shift*, calculates the shift and directly corrects
+   the data.
+ - The *Show Alignment Window* option, showing a window that allows for specification of the
+   shift and alignment methods, as well as offering the possibility to save calculated shifts
+   and load previously calculated shifts from a file.
+   It is also possible to enter shift values by hand.
+
+
+Once the *Alignment Window* is opened, the alignment method and the shift method must be specified.
+The alignment method specifies how the shift is calculated, while the shift method determines
+how the shift is applied to the data.
+
+The table shows three columns:
+
+ - The first one shows the plot legend of the data that will be
+   corrected by the shift method.
+ - The second column shows the plot legend from which the shift
+   is calculated.
+ - The third column shows the shift values calculated by the alignment method in
+   units of the plot windows x-axis.
+
+While columns one and two can not be edited, shift values
+can be entered by hand. Another way of setting the shift values is to load them from a existing
+\*.shift file using the Load button.
+
+Once the shift values are set, they can either be directly applied to the data present in the
+plot window, using the *Apply* button, or the data can be stored in memory. The latter options allow
+to use a reference signal recorded during the experiment, to determine the shift and then apply
+the shift values to a different set of data.
+
+.. note::
+
+  In order to match different sets of data to another, as necessary in the case of a
+  reference signal, the order in which the data is added to the plot window is crucial. If one
+  switches between two sets of data, where one set aligns the other one, it is highly encouraged
+  to consult the table in the *Alignment window* to check if every element in the two different
+  sets of data is assigned to its correct counterpart before applying the shift.
+
+
+If the data in the plot window is zoomed-in to a distinct feature, only this range of the data
+is used to calculate the shift.
+
+Methods used by the plug-in
+---------------------------
+
+Alignment methods are used to calculate the shift. Present methods include FFT, MAX, FIT and
+FIT DRV.
+
+*FFT*:
+
+    Uses the Fourier Transform of the curves to calculate their cross-correlation. The maximum
+    of the correlation is determined, and yields the shift value. This method is the default option.
+    Since it is not affected by the peak shape, it is fast and numerically robust.
+
+    .. note:: The shifts are given in real space values.
+
+*MAX*:
+
+    Determines the maximum of each curve. The shift is given by the differences in the x-position
+    of the maxima. Note that this method is highly vulnerable to noise in the data and spikes.
+
+*FIT*:
+
+    This method subtracts a background from the data using the SNIP algorithm and searches for peaks
+    in the data. For every curve, the single most pronounced feature is selected.
+    The peak is fitted by a Gaussian model. The shifts are then given by differences in the x-offsets
+    of the fitted Gaussians.
+
+*FIT DRV*:
+
+    Uses the same procedure as the FIT method. However the fit is applied to the first derivative of
+    the data. This method is only recommended for X-ray absorption data.
+
+Shift methods are used to apply the calculated shift to the data. Present methods include *Shift x-range*
+and *Inverse FFT shift*.
+
+*Shift x-range*:
+
+    This method adds the calculated shift value to every point.
+
+*Inverse FFT shift*:
+
+    Takes the Fourier Transform of a curve and multiplies the shift as a phase factor. The multiplication
+    of a phase factor in Fourier space translates to a shift in the x-range in real space. The shifted data
+    is given by the inverse Fourier transform.
+
+    .. note::
+
+        For this  process, the data needs to have a equidistant x-range. If this is not the case, the data
+        will be interpolated on a equidistant x-range. Due to the cyclic nature of the Fourier transform, this
+        method is recommended for data that has linear background.
+"""
 import numpy
+import logging
 import sys
 import traceback
 from PyMca5.PyMcaGui import PyMcaQt as qt
@@ -41,13 +145,15 @@ from PyMca5.PyMcaMath.fitting.SpecfitFuns import gauss
 from PyMca5.PyMcaMath.fitting import SpecfitFuns
 from os.path import join as pathjoin
 
+_logger = logging.getLogger(__name__)
+
 try:
     from PyMca5 import Plugin1DBase
 except ImportError:
-    print("WARNING:AlignmentScanPlugin import from somewhere else")
+    _logger.warning("WARNING:AlignmentScanPlugin import from somewhere else")
     from . import Plugin1DBase
 
-DEBUG = 0
+
 class AlignmentWidget(qt.QDialog):
 
     _storeCode = 2
@@ -257,8 +363,7 @@ class AlignmentWidget(qt.QDialog):
             return False
         if not str(filename).endswith('.shift'):
             filename += '.shift'
-        if DEBUG == 1:
-            print('saveOptions -- Filename: "%s"' % filename)
+        _logger.debug('saveOptions -- Filename: "%s"', filename)
         currentOrder = self.plugin.getOrder()
         outDict = ConfigDict.ConfigDict()
         llist, ddict = self.getDict()
@@ -459,8 +564,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
         Returns the legends of the curves in the plot winow
         in the order they were added.
         """
-        ret = [legend for (x,y,legend,info) in self._plotWindow.getAllCurves()]
-        return ret
+        return self.getAllCurves(just_legend=True)
 
     # BEGIN Alignment Methods
     def calculateShiftsFitDerivative(self):
@@ -483,8 +587,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             xmin = xmin0
         if xmax0 < xmax:
             xmax = xmax0
-        if DEBUG == 1:
-            print('calculateShiftsFit -- xmin = %.3f, xmax = %.3f'%(xmin, xmax))
+        _logger.debug('calculateShiftsFit -- xmin = %.3f, xmax = %.3f', xmin, xmax)
 
         # Get active curve
         activeCurve = self.getActiveCurve()
@@ -516,11 +619,11 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
                                     numpy.asarray([yp0, xp0, fwhm0]),
                                     xdata=x0[fitrange0],
                                     ydata=y0[fitrange0])
-        if DEBUG == 1:
-            if derivative:
-                print('calculateShiftsFit -- Results (Leg, PeakPos, Shift):')
-            else:
-                print('calculateShiftsFitDerivative -- Results (Leg, PeakPos, Shift):')
+
+        if derivative:
+            _logger.debug('calculateShiftsFit -- Results (Leg, PeakPos, Shift):')
+        else:
+            _logger.debug('calculateShiftsFitDerivative -- Results (Leg, PeakPos, Shift):')
         for x,y,legend,info in curves:
             idx = numpy.nonzero((xmin <= x) & (x <= xmax))[0]
             x = numpy.take(x, idx)
@@ -554,8 +657,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             key = legend
             retList.append(key)
             retDict[key] = shift
-            if DEBUG == 1:
-                  print( '\t%s\t%.3f\t%.3f'%(legend, fitp[1], shift))
+            _logger.debug('\t%s\t%.3f\t%.3f', legend, fitp[1], shift)
         return retList, retDict
 
     def calculateShiftsMax(self):
@@ -596,10 +698,9 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
 
         # Determine the index of maximum in active curve
         shift0 = numpy.argmax(y0)
-        if DEBUG == 1:
-            print('calculateShiftsMax -- Results:')
-            print('\targmax(y) shift')
-        for x,y,legend,info in curves:
+        _logger.debug('calculateShiftsMax -- Results:')
+        _logger.debug('\targmax(y) shift')
+        for x, y, legend, info in curves:
             idx = numpy.nonzero((xmin <= x) & (x <= xmax))[0]
             x = numpy.take(x, idx)
             y = numpy.take(y, idx)
@@ -609,8 +710,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             key = legend
             retList.append(key)
             retDict[key] = shift
-            if DEBUG == 1:
-                print('\t%d %.3f'%(x[shifty],shift))
+            _logger.debug('\t%d %.3f', x[shifty], shift)
         return retList, retDict
 
     def calculateShiftsFFT(self, portion=.95):
@@ -630,8 +730,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             xmin = xmin0
         if xmax0 < xmax:
             xmax = xmax0
-        if DEBUG == 1:
-            print('calculateShiftsFFT -- xmin = %.3f, xmax = %.3f'%(xmin, xmax))
+        _logger.debug('calculateShiftsFFT -- xmin = %.3f, xmax = %.3f', xmin, xmax)
 
         # Get active curve
         activeCurve = self.getActiveCurve()
@@ -651,8 +750,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
         y0 = numpy.take(y0, idx)
 
         fft0 = numpy.fft.fft(y0)
-        if DEBUG == 1:
-            print('calculateShiftsFFT -- results (Legend len(idx) shift):')
+        _logger.debug('calculateShiftsFFT -- results (Legend len(idx) shift):')
         for x,y,legend,info in curves:
             idx = numpy.nonzero((x >= xmin) & (x <= xmax))[0]
             x = numpy.take(x, idx)
@@ -682,8 +780,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             key = legend
             retList.append(key)
             retDict[key] = shift
-            if DEBUG == 1:
-                print('\t%s\t%d\t%f'%(legend,len(idx),shift))
+            _logger.debug('\t%s\t%d\t%f', legend, len(idx), shift)
         return retList, retDict
     # END Alignment Methods
 
@@ -724,19 +821,16 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             if msg.exec_() != qt.QMessageBox.Ok:
                 return False
 
-        if DEBUG == 1:
-            print('applyShifts -- Shifting ...')
+        _logger.debug('applyShifts -- Shifting ...')
         for idx, (x, y, legend, info) in enumerate(curves):
             shift = self.shiftDict[legend]
 
             if shift is None:
-                if DEBUG == 1:
-                    print('\tCurve \'%s\' not found in shiftDict\n%s'\
-                          %(legend,str(self.shiftDict)))
+                _logger.debug('\tCurve \'%s\' not found in shiftDict\n%s',
+                              legend, str(self.shiftDict))
                 continue
             if shift == float('NaN'):
-                if DEBUG == 1:
-                    print('\tCurve \'%s\' has NaN shift'%legend)
+                _logger.debug('\tCurve \'%s\' has NaN shift', legend)
                 continue
 
             # Limit shift to zoomed in area
@@ -753,8 +847,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             else:
                 replot = False
             # Check if scan number is adopted by new curve
-            if DEBUG == 1:
-                print('\'%s\' -- shifts -> \'%s\' by %f'%(self.shiftList[idx], legend, shift))
+            _logger.debug('\'%s\' -- shifts -> \'%s\' by %f', self.shiftList[idx], legend, shift)
             #selectionlegend = info.get('selectionlegend', legend)
             selectionlegend = legend
             self.addCurve(xShifted, yShifted,
@@ -814,8 +907,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
         Method receives methodName from AlignmentWidget
         instance and assigns the according shift method.
         """
-        if DEBUG == 1:
-            print('setShiftMethod -- %s'%methodName)
+        _logger.debug('setShiftMethod -- %s', methodName)
         methodName = str(methodName)
         if methodName == 'Inverse FFT shift':
             self.shiftMethod = self.fftShift
@@ -830,8 +922,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
         Method receives methodName from AlignmentWidget
         instance and assigns the according alignment method.
         """
-        if DEBUG == 1:
-            print('setAlignmentMethod -- %s'%methodName)
+        _logger.debug('setAlignmentMethod -- %s', methodName)
         methodName = str(methodName)
         if methodName == 'FFT':
             self.alignmentMethod = self.calculateShiftsFFT
@@ -851,7 +942,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
         is strictly monotonically increasing.
         Conserves curves legend and info dictionary.
         """
-        curves = Plugin1DBase.Plugin1DBase.getAllCurves(self)
+        curves = Plugin1DBase.Plugin1DBase.getAllCurves(self, just_legend=just_legend)
         if just_legend:
             return curves
 
@@ -895,8 +986,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
         """
         curves = self.getAllCurves()
         if len(curves) < 1:
-            if DEBUG == 1:
-                print('interpolate -- no curves present')
+            _logger.debug('interpolate -- no curves present')
             raise ValueError("At least 1 curve needed")
             return
 
@@ -961,9 +1051,8 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
                     xmin0 = xmin
                 if xmax > xmax0:
                     xmax0 = xmax
-        if DEBUG == 1:
-            print('getXLimits -- overlap = %s, xmin = %.3f, xmax =%.3f'\
-                  %(overlap,xmin0,xmax0))
+        _logger.debug('getXLimits -- overlap = %s, xmin = %.3f, xmax =%.3f',
+                      overlap, xmin0, xmax0)
         return xmin0, xmax0
 
     def normalize(self, y):
@@ -1020,12 +1109,10 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             # Extract highest peak
             sortIdx = y[peakIdx].argsort()[-1]
         except IndexError:
-            if DEBUG == 1:
-                print('No peaks found..')
+            _logger.debug('No peaks found..')
             return None
         except SystemError:
-            if DEBUG == 1:
-                print('Peak search failed. Continue with y maximum')
+            _logger.debug('Peak search failed. Continue with y maximum')
             peakIdx = [ybg.argmax()]
             sortIdx = 0
         xpeak = float(x[peakIdx][sortIdx])
@@ -1065,8 +1152,7 @@ class AdvancedAlignmentScanPlugin(Plugin1DBase.Plugin1DBase):
             msg.setWindowTitle('Alignment Scan Error')
             msg.setText('No help file found.')
             msg.exec_()
-            if DEBUG == 1:
-                print('XMCDWindow -- init: Unable to read help file')
+            _logger.debug('XMCDWindow -- init: Unable to read help file')
             self.helpFileBrowser = None
         if self.helpFileBrowser is not None:
             self.helpFileBrowser.show()
@@ -1084,7 +1170,7 @@ if __name__ == "__main__":
     a.show()
 
     x = numpy.arange(250, 750, 2, dtype=float)
-    y1 = 1.0 + 50.0 * numpy.exp(-0.001*(x-500)**2) + 2.*numpy.random.random(250.)
-    y2 = 1.0 + 20.5 * numpy.exp(-0.005*(x-600)**2) + 2.*numpy.random.random(250.)
+    y1 = 1.0 + 50.0 * numpy.exp(-0.001*(x-500)**2) + 2.*numpy.random.random(250)
+    y2 = 1.0 + 20.5 * numpy.exp(-0.005*(x-600)**2) + 2.*numpy.random.random(250)
 
     app.exec_()
