@@ -30,8 +30,12 @@
 # If everything works well one should find a directory in the build
 # directory that contains the files needed to run the PyMca without Python
 #
-from cx_Freeze import setup, Executable
+from cx_Freeze import setup, Executable, version
+if not version.startswith('4'):
+    raise RuntimeError("At this point only cx_Freeze 4.x.x supported")
 import sys
+if sys.version_info >= (3,):
+    raise RuntimeError("At this point only Python 2 supported")
 import os
 import glob
 import cx_Freeze.hooks as _hooks
@@ -61,7 +65,11 @@ def load_PyQt4_Qt(finder, module):
         except ImportError:
             pass
 
+def dummy(*var, **kw):
+    pass
+
 _hooks.load_PyQt4_Qt = load_PyQt4_Qt
+_hooks.copy_qt_plugins = dummy
 
 
 PyMcaInstallationDir = "build"
@@ -79,7 +87,8 @@ if sys.platform == 'win32':
             print("Error building PyMca")
             sys.exit(1)
 
-cmd = "python setup.py install --install-lib %s --distutils" % PyMcaInstallationDir
+cmd = "python setup.py install --install-lib %s --install-data %s --distutils" % \
+              (PyMcaInstallationDir, PyMcaInstallationDir)
 if os.system(cmd):
     print("Error building PyMca")
     sys.exit(1)
@@ -114,13 +123,10 @@ for pluginSet in glob.glob(os.path.join(pluginsDir,'*')):
 
 #I should use somehow absolute import ...
 sys.path = [PyMcaInstallationDir] + sys.path[1:]
-try:
-    import ctypes
-    import OpenGL
-    from PyMca5 import Object3D
-    OBJECT3D = True
-except:
-    OBJECT3D = False
+import ctypes
+import OpenGL
+from PyMca5 import Object3D
+OBJECT3D = True
 
 try:
     import scipy
@@ -133,17 +139,13 @@ except:
 # For the time being I leave SciPy out
 SCIPY = False
 
-try:
-    import matplotlib
-    MATPLOTLIB = True
-except ImportError:
-    MATPLOTLIB = False
+import matplotlib
+MATPLOTLIB = True
 
 try:
     import pyopencl
     OPENCL = True
-    from PyMca5.PyMcaMath import sift
-except :
+except:
     OPENCL = False
 
 if sys.platform.lower().startswith("linux"):
@@ -157,60 +159,56 @@ except ImportError:
     MDP = False
 
 H5PY_SPECIAL = False
-try:
-    import h5py
-    if h5py.version.version < '1.2.0':
-        includes = ['h5py._extras']
-    elif h5py.version.version < '1.3.0':
-        includes = ['h5py._stub', 'h5py._sync', 'h5py.utils']
-    elif h5py.version.version < '2.0.0':
-        includes = ['h5py._extras', 'h5py._stub', 'h5py.utils',
-                    'h5py._conv', 'h5py._proxy']
-    else:
-        H5PY_SPECIAL = True
-        includes = []
-except:
+import h5py
+if h5py.version.version < '1.2.0':
+    includes = ['h5py._extras']
+elif h5py.version.version < '1.3.0':
+    includes = ['h5py._stub', 'h5py._sync', 'h5py.utils']
+elif h5py.version.version < '2.0.0':
+    includes = ['h5py._extras', 'h5py._stub', 'h5py.utils',
+                'h5py._conv', 'h5py._proxy']
+else:
+    H5PY_SPECIAL = True
     includes = []
 
-try:
-    import fisx
-    FISX = True
-except ImportError:
-    print("Please install fisx module prior to freeze PyMca")
-    FISX = False
-    raise
-
-try:
-    import hdf5plugin
-except ImportError:
-    pass
-
+import fisx
+FISX = True
 
 #some standard encodings
 includes.append('encodings.ascii')
 includes.append('encodings.utf_8')
 includes.append('encodings.latin_1')
 import PyMca5
+import hdf5plugin
+import silx
+import pkg_resources
+SILX = True
 
 special_modules = [os.path.dirname(PyMca5.__file__),
-                  os.path.dirname(ctypes.__file__)]
+                   os.path.dirname(matplotlib.__file__),
+                   os.path.dirname(ctypes.__file__),
+                   os.path.dirname(fisx.__file__),
+                   os.path.dirname(hdf5plugin.__file__),
+                   os.path.dirname(silx.__file__),
+                   os.path.dirname(pkg_resources.__file__)]
 
-if sys.platform == "win32":
-    try:
-        import hdf5plugin
-        special_modules.append(os.path.dirname(hdf5plugin.__file__))
-    except ImportError:
-        print("Please install hdf5plugin prior to freeze PyMca")
-        raise
+try:
+    import tomogui
+    special_modules.append(os.path.dirname(tomogui.__file__))
+    import freeart
+    special_modules.append(os.path.dirname(freeart.__file__))
+except:
+    pass
 
 
 excludes = ["Tkinter", "tkinter",
             'tcl','_tkagg', 'Tkconstants',
-            "scipy", "Numeric", "numarray"]
+            "scipy", "Numeric", "numarray", "PyQt5"]
 
 try:
     import IPython
     if IPython.__version__.startswith('2'):
+        # this works with IPython 2.4.1
         special_modules.append(os.path.dirname(IPython.__file__))
         includes.append("colorsys")
         import pygments
@@ -236,16 +234,21 @@ if H5PY_SPECIAL:
     special_modules.append(os.path.dirname(h5py.__file__))
 if OPENCL:
     special_modules.append(os.path.dirname(pyopencl.__file__))
-    includes.append("pytools")
+    import mako
+    import cffi
+    import pytools
+    special_modules.append(os.path.dirname(mako.__file__))
+    special_modules.append(os.path.dirname(cffi.__file__))
+    special_modules.append(os.path.dirname(pytools.__file__))
+    #includes.append("pytools")
     includes.append("decorator")
 else:
     excludes.append("pyopencl")
+
 if MDP:
     #mdp versions above 2.5 need special treatment
     if mdp.__version__  > '2.5':
         special_modules.append(os.path.dirname(mdp.__file__))
-if MATPLOTLIB:
-    special_modules.append(os.path.dirname(matplotlib.__file__))
 if SCIPY:
     special_modules.append(os.path.dirname(scipy.__file__))
 
@@ -254,11 +257,8 @@ if OBJECT3D:
     excludes.append("OpenGL")
     special_modules.append(os.path.dirname(OpenGL.__file__))
 
-if FISX:
-    special_modules.append(os.path.dirname(fisx.__file__))
-
 for f in special_modules:
-        include_files.append((f,os.path.basename(f)))
+    include_files.append((f, os.path.basename(f)))
 
 for f in ['qt', 'qttable', 'qtcanvas', 'Qwt5']:
     excludes.append(f)
@@ -351,10 +351,30 @@ setup(
                        ),
         executables = executables)
 
+if SILX:
+    # silx gui._qt module needs to be patched to get rid of uic
+    initFile = os.path.join(install_dir, "silx", "gui", "qt", "_qt.py")
+    print("###################################################################")
+    print("Patching silx file")
+    print(initFile)
+    print("###################################################################")
+    f = open(initFile, "r")
+    content = f.readlines()
+    f.close()
+    f = open(initFile, "w")
+    for line in content:
+        if ("PyQt4.uic" in line) or ("PyQt5.uic" in line):
+            continue
+        f.write(line)
+    f.close()
 
 if OPENCL:
     # pyopencl __init__.py needs to be patched
     initFile = os.path.join(install_dir, "pyopencl", "__init__.py")
+    print("###################################################################")
+    print("Patching pyopencl file")
+    print(initFile)
+    print("###################################################################")
     f = open(initFile, "r")
     content = f.readlines()
     f.close()
@@ -467,9 +487,23 @@ else:
     if DEBUG:
         print("NO PROBLEM")
 
+# cleanup binary modules already added within packages
+files = glob.glob(os.path.join(install_dir, "*"))
+for fname in files:
+    for module in ["PyMca5", "matplotlib", "fisx", "silx",
+                   "h5py", "hdf5", "freeart"]:
+        basename = os.path.basename(fname)
+        if basename.startswith(module + "_"):
+            os.remove(fname)
+            print("DELETING %s" % fname)
+        elif basename.startswith(module + "."):
+            os.remove(fname)
+            print("DELETING %s" % fname)
+
 if os.path.exists('bin'):
     for f in glob.glob(os.path.join('bin','*')):
         os.remove(f)
+        print("DELETING %s" % f)
     os.rmdir('bin')
 
 if not SCIPY:
@@ -479,6 +513,26 @@ if not SCIPY:
             print("Deleting plugin %s" % plugin)
             os.remove(plugin)
 
+nsis = os.path.join("\Program Files (x86)", "NSIS", "makensis.exe")
+if sys.platform.startswith("win") and os.path.exists(nsis):
+    # check if we can perform the packaging
+    outFile = "nsisscript.nsi"
+    f = open("nsisscript.nsi.in", "rb")
+    content = f.readlines()
+    f.close()
+    if os.path.exists(outFile):
+        os.remove(outFile)
+    pymcaexe = "pymca%s-win64.exe" % PyMca5.version()
+    if os.path.exists(pymcaexe):
+        os.remove(pymcaexe)
+    f = open(outFile, "wb")
+    for line in content:
+        line = line.replace("__VERSION__", PyMca5.version())
+        f.write(line)
+    f.close()
+    cmd = '"%s" %s' % (nsis, outFile)
+    print(cmd)
+    os.system(cmd)
 sys.exit(0)
 
 ########################## WHAT FOLLOWS IS UNUSED CODE ################
